
! ---------------------------------------------------------------- smcpadf
      subroutine smcpadf(y, npts_in, sps, nlz, ntz, 
     :  replace_discarded_data_with_zeros, 
     :  taper_beginning, taper_end, 
     :  npts_out, izcl, izct, 
     :  remove_mean_before, remove_mean_after, 
     :  zcross_l, use_first, zcross_t, use_last,
     :  comments, ncomments)    

* Here is the padding action taken for some combinations of input parameters:
*
* zcross_l  use_first  pad action (similar for zcross_t, use_last)
*     T         T         pad with 0.0
*     T         F         pad with 0.0
*     F         T         pad with work(1)
*     F         F         pad with 0.0
 
* Dates: 06/04/04 - Replaced tlz, ttz with nlz, ntz.  Also, allow the 
*                   discarded data to be replaced with zeros (thus 
*                   preserving the length of the input time series).
*        03/13/06 - Remove f_smc from smcpadf
*        11/03/06 - If zcross_l or zcross_t = F and use_first or 
*                   use_last = T, pad with values equal to the last points.
*                   Allow this option to deal with cases such as filtering
*                   a displacement time series with a low-frequency drift, 
*                   which will have a step offset if pad with zeros at the
*                   end.  Be careful in in using this option for the first
*                   points, however (in a future version I may eliminate
*                   the option for the beginning, as generally the leading
*                   pad should be zero).  The need for this option arose
*                   when I tried to filter a displacement obtained by 
*                   integrating a zeroth-order corrected acceleration.
*                   The results were different than filtering the 
*                   acceleration before integration.
*        01/24/07 - Store the snipped portion of y_in in a work array.  This
*                   is to be sure that I am not overwriting the input data
*                   when shifting the time series.  Probably the existing
*                   code was OK, but on reading it some years after writing
*                   it, I find it difficult to follow.  It is much better to
*                   make things a bit less efficient, but easier to follow.
*        01/24/07 - Added option to taper end of time series before adding 
*                   trailing zero pad.
*        07/15/07 - Added option to taper beginning of time series before 
*                   adding leading zero pad.
*        07/17/07 - Corrected a subtle error:  the names for the input 
*                   parameters taper_beginning and taper_end were being used
*                   for the variable containing the taper, and thus their
*                   values were being reset to the last taper value.   I now
*                   use "taper" as the variable containing the taper value.
*        03/24/08 - Write leading zpads as a time as well as n points.
*        01/18/10 - Add comments above about padding
!        04/04/10 - Correct error that pad_value_l and pad_value_t could be
!                   undefined in some cases.

  
      logical remove_mean_before, remove_mean_after,
     :  zcross_l, use_first, zcross_t, use_last, 
     :  replace_discarded_data_with_zeros

      real y(*)
      character comments(*)*80

      real work(:)
      allocatable :: work

      parameter (small=1.0e-10)
!      parameter (small=1.0e-05)

      pi = 4.0*atan(1.0)

!* DEBUG
!      write(*,'(a)') ' Inside smcpadf:'
!
!      write(*,'(a)') ' nlz, ntz = '
!      write(*, *)      nlz, ntz
!
!      write(*,'(a)') ' zcross_l, zcross_t = '
!      write(*, *)      zcross_l, zcross_t
!
!      write(*,'(a)') ' use_first, use_last = '
!      write(*, *)      use_first, use_last
!
!      write(*,'(a,1x,l1)') 
!     :      ' remove_mean_before = ', remove_mean_before
!
!      write(*,'(a,1x,l1)') 
!     :      ' remove_mean_after = ', remove_mean_after
!
      if (remove_mean_before) then        
        n_start = 1
        n_stop = npts_in
        icorr_cumavg_before = 0
        call Get_Avg(y, n_start, n_stop, npts_in, 
     :                   sps, icorr_cumavg_before, avg_before)
        do i = n_start, n_stop
          y(i) = y(i) - avg_before
        end do
        n_start_before = n_start
        n_stop_before = n_stop
      end if

* Get peak motion:
      
      call mnmaxidx(y, 1, npts_in, 1,
     :             ymin, ymax, indx_min, indx_max) 
      peak_mtn = ymax
      if (abs(ymin) .gt. abs(ymax)) peak_mtn = ymin

* Define fraction small of peak:
      smidge = small * abs(peak_mtn)


      if (zcross_l) then ! Find first zero crossing
        izcl = -1
        if (abs(y(1)) .lt. smidge) then
          izcl = 1
        else
          do i = 2, npts_in
            if (y(i)*y(1) .lt. 0.0) then   ! a zero crossing has been found
* Now determine if the point just before the zero crossing is smaller in 
* amplitude than the point after.  If so, let it be the first point unless 
* the first value is smaller than this value, in which case y(1) is the 
* first value.
              if (abs(y(i)) .lt. abs(y(i-1))) then
                izcl = i
              else
                izcl = i - 1
                if (use_first) then
                  if (abs(y(i-1)) .gt. abs(y(1))) then
                    izcl = 1
                  end if
                end if
              end if
              go to 2345
            end if
          end do
2345      continue
        end if
      else
        izcl = 1
      end if
      if (zcross_t) then ! Find last zero crossing
        izct = -1
        if (abs(y(npts_in)) .lt. smidge) then
          izct = npts_in
        else
          do i = npts_in -1, 1, -1
            if (y(i)*y(npts_in) .lt. 0.0) then ! a zero crossing has been found
* Now determine if the point just after the zero crossing is smaller in 
* amplitude than the point before.  If so, let it be the last point unless 
* the last value is smaller than this value, in which case y(npts_in) is the 
* last value.
              if (abs(y(i)) .lt. abs(y(i+1))) then
                izct = i
              else
                izct = i+1
                if (use_last) then
                  if (abs(y(i+1)) .gt. abs(y(npts_in))) then
                    izct = npts_in
                  end if
                end if
              end if
              go to 5432
            end if
          end do
5432      continue
        end if
      else
        izct = npts_in
      end if

!      write(*,'(a)') ' izcl, izct = '
!      write(*,*)       izcl, izct

* izcl, izct = index in input smc file of zero crossings

* April's code is more compact, but I think mine is easier to read        
            
      if (replace_discarded_data_with_zeros) then
        nlznew = nlz + izcl - 1
        ntznew = ntz + npts_in - izct
      else
        nlznew = nlz
        ntznew = ntz
      end if

* Allocate a work array and save the snipped portion in this array

      npts_snip = izct - izcl + 1  ! number of original points between zeros
      allocate (work(npts_snip)) 
      do i = 1, npts_snip
        work(i) = y(i+izcl-1)
      end do
      
* Compute new npts_out:

      npts_out = npts_snip + nlznew + ntznew

* Indices of original data

        n_start = nlznew + 1
        n_stop = npts_snip + nlznew

* Compute and remove mean if specified:

      if (remove_mean_after) then        
        n_start4avg = 1
        n_stop4avg = npts_snip 
        icorr_cumavg_after = 1

        call Get_Avg(work, n_start4avg, n_stop4avg, npts_snip, 
     :                   sps, icorr_cumavg_after, avg_after)
        do i = 1, npts_snip
          work(i) = work(i) - avg_after
        end do
        n_start_after = n_start
        n_stop_after = n_stop
      end if

* Taper beginning, if desired:

      ntaper_beginning = taper_beginning*sps
      
      if (ntaper_beginning .gt. 0) then
      
!* DEBUG
!        print *,' ntaper_beginning = ', ntaper_beginning
!* DEBUG
      
        do i = 1, ntaper_beginning + 1
        
           ishift = i - (1 - ntaper_beginning)
           arg = pi*float(ishift)/float(ntaper_beginning)
           taper = 0.5*(1.0+cos(arg))
           
           work(i) = taper * work(i)
           
        end do
        
      end if



* Taper end, if desired:

      ntaper_end = taper_end*sps
      
      if (ntaper_end .gt. 0) then
      
!* DEBUG
!        print *,' ntaper_end = ', ntaper_end  
!* DEBUG
      
        do i = npts_snip - ntaper_end, npts_snip
        
           ishift = i - (npts_snip - ntaper_end)
           arg = pi*float(ishift)/float(ntaper_end)
           taper = 0.5*(1.0+cos(arg))
           
           work(i) = taper * work(i)
           
        end do
        
      end if



* Store work in shifted location in y

      do i = 1, npts_snip
        y(i + nlznew) = work(i)
      end do
      
* And pad with leading zeros:

      pad_value_l = 0.0
      if ( nlznew .gt. 0) then
        if ((.not. zcross_l) .and. use_first) then
          pad_value_l = work(1)
        else
          pad_value_l = 0.0
        end if
        
        do i = 1, nlznew
          y(i) = pad_value_l
        end do
      end if

* And pad with trailing zeros:

      pad_value_t = 0.0
      if ( ntznew .gt. 0) then
        if ((.not. zcross_t) .and. use_last) then
          pad_value_t = work(npts_snip)
        else
          pad_value_t = 0.0
        end if

        do i = 1, ntznew
          y(npts_snip + nlznew + i) = pad_value_t
        end do
      end if

      deallocate(work)

* Store  comments:
      ncomments = ncomments + 1
      write(comments(ncomments),'(a)') 
     :     '| Added pads:'
      if (remove_mean_before) then        
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,i1,a,1x,1p,e11.4)') 
     :   '| Using icorr_cumavg = ', icorr_cumavg_before, 
     :   ', determined and removed the mean', avg_before
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,i5,1x,a,1x,i5,a)') 
     :   '| between indices ', n_start_before, ' and ',
     :        n_stop_before, ' before padding with pad_value'
      else
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,i1,a,1x,1p,e11.4)') 
     :   '| No mean removed before adding pad' 
      end if
      if (remove_mean_after) then        
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,i1,a,1x,1p,e11.4)') 
     :   '| Using icorr_cumavg = ', icorr_cumavg_after, 
     :   ', determined and removed the mean', avg_after
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,i5,1x,a,1x,i5,a)') 
     :   '| between indices ', n_start_after, ' and ',
     :        n_stop_after, ' after padding with pad_value'
      else
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,i1,a,1x,1p,e11.4)') 
     :   '| No mean removed after adding pads' 
      end if
      
      if (ntaper_beginning .gt. 0) then
!* DEBUG
!        print *,' In write comments:'
!        print *,' ntaper_beginning = ', ntaper_beginning  
!* DEBUG
      
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,i7,a)')
     :     '| Taper beginning ', ntaper_beginning,
     :     ' points before adding leading pad'
      end if
          
      if (ntaper_end .gt. 0) then
!* DEBUG
!        print *,' In write comments:'
!        print *,' ntaper_end = ', ntaper_end 
!* DEBUG
      
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,i7,a)')
     :     '| Taper last ', ntaper_end,
     :     ' points before adding trailing pad'
      end if
          
      ncomments = ncomments + 1
      if (replace_discarded_data_with_zeros) then
        write(comments(ncomments),'(a,1x,i7,a)') 
     :     '| The program replaced ', izcl - 1, 
     :      ' points from start of record with pad value'
      else
        write(comments(ncomments),'(a,1x,i7,a)') 
     :     '| The program discarded ', izcl - 1, 
     :      ' points from the start of the record'
      end if
      ncomments = ncomments + 1
      write(comments(ncomments),'(a,1x,i7,a,
     :                            f7.3,a)') 
     :     '|   and added ', nlz, ' leading values ( ', 
     :                     float(nlz)/sps, ' s), using'
      ncomments = ncomments + 1
      write(comments(ncomments),'(a,1x,l1,a,1x,l1)') 
     :     '|   zcross_l = ', zcross_l,
     :        ' and use_first = ', use_first
      ncomments = ncomments + 1
      write(comments(ncomments),'(a,1x,1p, e10.3)') 
     :     '|   and pad_value = ', pad_value_l
      ncomments = ncomments + 1
      if (replace_discarded_data_with_zeros) then
        write(comments(ncomments),'(a,1x,i7,a)') 
     :     '| The program replaced ', npts_in - izct, 
     :      ' points from end of record with pad_value'
      else
        write(comments(ncomments),'(a,1x,i7,a)') 
     :     '| The program discarded ', npts_in - izct, 
     :      ' points from the end of the record'
      end if
      ncomments = ncomments + 1
      write(comments(ncomments),'(a,1x,i7,a)') 
     :     '|   and added ', ntz, ' trailing values, using'
      ncomments = ncomments + 1
      write(comments(ncomments),'(a,1x,l1,a,1x,l1)') 
     :     '|   zcross_t = ', zcross_t,
     :        ' and use_last = ', use_last
      ncomments = ncomments + 1
      write(comments(ncomments),'(a,1x,1p, e10.3)') 
     :     '|   and pad_value = ', pad_value_t
      ncomments = ncomments + 1
      write(comments(ncomments),'(a,1x,i7)') 
     :     '| The total number of points in the input time'//
     :        ' series = ', npts_in
      ncomments = ncomments + 1
      write(comments(ncomments),'(a,1x,i7)') 
     :     '| The total number of points in the '//
     :       'pad-stripped series = ', npts_out - nlz - ntz

      return
      end
! ---------------------------------------------------------------- smcpadf



! ----------------------------------------------------------------- begin RS_Calc
      subroutine rs_calc(acc, na, omega, damp, dt,
     :                   v0, d0, use_straightline_interp, rd, rv, aa)
        
!-----------------------------------------------------------------------
! This version does not return response time series.

! NOTE: "interp_acc" in the name of the previous version (see below)
! refers to the straightline interpolation
! done in subroutine icmpmx. The name was chosen before the sinc interpolation
! (Boore and Goulet) was used in computing response spectra (in smc2rs, as revised in
! February 2018) and RotDxx (in smc2psa_rot_gmrot_interp_acc_rot_osc_ts.for).
! sinc interpolation is accomplished by first interpolating a time series
! using interpolate_time_series_using_fork, and then passing that interpolated
! time series into rs_calc.  This is done in smc2rs.for.


! Dates: 02/09/18 - A revised version of rscalc_interp_acc.for; I intend for this 
!                   subroutine to be the only one called by other programs.  I added
!                   a variable to skip the straightline interpolation if desired
!                   (this interpolation is probably not needed if sinc interpolation
!                   has been used to resample the original time series with more sps),
!                   and I include the subroutines rdrvaa and icmpmx via include statements.
!        02/27/18 - Correct error in assigning skip_straightline_interp in smc2rs (it
!                   was set to T when the input from the control file asking
!                   if linear interpolation was to be used was set to Y). The logical 
!                   way to do this is to change skip_straightline_interp to use_straightline_interp.
!                   This requires changes to a number of routines that use rs_calc, but
!                   is the right thing to do.
!        08/09/18 - No longer bring in rdrvaa and icmpmx using an include statement, because 
!                   this is a problem in make_td_subs_file.bat.
!                 - I decide not to do this, because it will have reverberations when compiling
!                   various programs in \forprogs\.  Instead, I added a comment that
!                   SMSIM and EXSIM users should edit the include statement as needed.
!                   


      implicit none

      real(4) :: acc(*), omega, damp, dt, 
     :       rd, rv, aa, d0, v0 
      
      integer :: na, kg, kug, npr
      
      logical use_straightline_interp


      real(8) :: ug(:), pr, damp_dble, dt_dble, z(3)

      real(8) :: w, twopi
      
      allocatable :: ug
      
      integer :: i, nn

      dt_dble = dble(dt)
      damp_dble = dble(damp)
      
      kg = na
      
      allocate(ug(na))

          do i=1,kg
            ug(i) = dble(acc(i))
          enddo
!...
!... Compute response spectra
!...
           kug=kg-1
           
      w = dble(omega)
      
      twopi = 4.0d0*dasin(1.0d0)
      
      pr = twopi/w

      if (use_straightline_interp) then
        if(dt_dble == 0.0d0 .or. pr < 10.0d0*dt_dble) then
          call icmpmx(kug, ug, dt_dble, pr, w, damp_dble, z)
          rd = sngl(z(1))
          rv = sngl(z(2))
          aa = sngl(z(3))
        else
          call rdrvaa(acc,kg,omega,damp,dt,rd,rv,aa, d0, v0)
        endif
      else
        call rdrvaa(acc,kg,omega,damp,dt,rd,rv,aa, d0, v0)
      end if
            
      deallocate(ug)

      return
      end

      include 'icmpmx.for'
      include 'rdrvaa.for'

  
!      include '\forprogs\icmpmx.for'
!      include '\forprogs\rdrvaa.for'

! NOTE: users of my program might need to edit the include statement to make sure that it 
! points to the folder with the subroutines, or comment out these lines if the subroutines
! are included in the subroutine file, such as in td_subs.for for SMSIM

! I think a work around is to not specify \forprogs and copy in icmpmx and rdrvaa into the SMSIM root when
! running \smsim\make_td_subs_file.bat

! ----------------------------------------------------------------- end RS_Calc

 


! ----------------------------------------------------- interpolate_time_series_using_fork
      subroutine interpolate_time_series_using_fork(
     : y_in, sps_in, npts_in, 
     : interpolate_factor,
     : y_out, sps_out, npts_out
     : )


! Interpolates an smc file to finer spacing, using an FFT with zeroes
! added after the Nyquist.

! Dates: 09/29/12 - Written by D.M. Boore, patterned after smc_interpolate_time_series_using_fft.
!                 - This version does not allow for linear resampling after interpolation;
!                   for that, see smcintrp.for.
!                 - Return input time series if interpolate_factor = 1
!        02/09/18 - Redefine npts_out to be the length of y_out.
 
      implicit none

      real y_in(*), y_out(*) 
      integer npts_in, npts_out, npw2
      real sps_in, sps_out

      integer :: interpolate_factor, i, j, npw2_in, n4ts_alloc,
     :           n_nyq_in, npw2_intrp

      real(4) :: sps_intrp, scale, anorm, signnpw2, df 
      

 
!
! Dimension and declaration statements:

      complex cx_in(:), cx_intrp(:)
      allocatable :: cx_in, cx_intrp
 
      if (interpolate_factor == 1) then
        npts_out = npts_in
        sps_out = sps_in
        do i = 1, npts_out
          y_out(i) = y_in(i)
        end do
        return
      end if

! Check if interpolate_factor is a power of 2: 
       if (modulo(alog(real(interpolate_factor)), alog(2.0)) /= 0) then
         print *,' interpolate_factor= ', 
     :              interpolate_factor, ' not a power of 2; QUIT!!'
         stop
       end if
 
 
! Compute the Fourier spectra:

! Fill working array with input data:

      signnpw2 = +1.0
      call get_npw2(npts_in,signnpw2,npw2_in)
      
      n4ts_alloc = max0(npts_in, npw2_in)
      allocate( cx_in(npw2_in) )
 
      do i = 1, npw2_in
        cx_in(i) = 0.0
      end do

      do i = 1, npts_in
        cx_in(i) = cmplx(y_in(i), 0.0)
      end do

      df = sps_in/float(npw2_in)
      
      n_nyq_in = npw2_in/2 + 1

! FFT to get spectrum

!      call forkdp(npw2_in,cx_in,-1.)
      call fork(npw2_in,cx_in,-1.)

! Fill new complex array, inserting zeros

      npw2_intrp = npw2_in * interpolate_factor
      sps_intrp = sps_in * real(interpolate_factor)
      sps_out = sps_intrp
      scale = sps_intrp/sps_in
      anorm = sqrt(scale)
      npts_out =int( scale * float(npts_in) )
      
      
      allocate( cx_intrp(npw2_intrp) )
 
      do j = 1, npw2_intrp
        cx_intrp(j) = 0.0
      end do

      do j = 1, n_nyq_in
        cx_intrp(j)= cx_in(j)
      end do

      do j = 1, n_nyq_in - 2
        cx_intrp(npw2_intrp+1-j)= cx_in(npw2_in+1-j)
      end do

      deallocate( cx_in)

! Inverse FFT:

!      call forkdp(npw2_intrp,cx_intrp,+1.)
      call fork(npw2_intrp,cx_intrp,+1.)

! Scale for different lengths and save only the points up to the old last point:

      do j = 1, npts_out
        y_out(j) = anorm * real(cx_intrp(j))
      end do

      deallocate( cx_intrp)
      
  
      return
      end
! ----------------------------------------------------- interpolate_time_series_using_fork

 

! ----------------------------- BEGIN FORK --------------------------
      SUBROUTINE FORK(LX,CX,SIGNI)
! FAST FOURIER                                  2/15/69
!                          LX
!    CX(K) = SQRT(1.0/LX)* SUM (CX(J)*EXP(2*PI*SIGNI*I*(J-1)*(K-1)/LX))
!                          J=1                        FOR K=1,2,...,LX
!
!  THE SCALING BETWEEN FFT AND EQUIVALENT CONTINUUM OUTPUTS
!  IS AS FOLLOWS.
!
!
!     GOING FROM TIME TO FREQUENCY:
!             F(W)=DT*SQRT(LX)*CX(K)
!
!                  WHERE W(K)=2.0*PI*(K-1)*DF

!                  and    DF = 1/(LX*DT)
!
!
!     GOING FROM FREQUENCY TO TIME, WHERE THE FREQUENCY
!     SPECTRUM IS GIVEN BY THE DIGITIZED CONTINUUM SPECTRUM:
!
!             F(T)=DF*SQRT(LX)*CX(K)
!
!                  WHERE T(K)=(K-1)*DT
!
!
!  THE RESULT OF THE SEQUENCE...TIME TO FREQUENCY,POSSIBLE MODIFICATIONS
!  OF THE SPECTRUM (FOR FILTERING,ETC.), BACK TO TIME...
!  REQUIRES NO SCALING.
!
!
!  THIS VERSION HAS A SLIGHT MODIFICATION TO SAVE SOME TIME...
!  IT TAKES THE FACTOR 3.1415926*SIGNI/L OUTSIDE A DO LOOP (D.BOORE 12/8
!  FOLLOWING A SUGGESTION BY HENRY SWANGER).
!

! Some brief notes on usage:

! "signi" is a real variable and should be called either with the value "+1.0"
! of "-1.0".  The particular value used depends on the conventions being used
! in the application (e.g., see Aki and Richards, 1980, Box 5.2, pp. 129--130).

! Time to frequency:
! In calling routine,
! 
!       do i = 1, lx
!         cx(i) = CMPLX(y(i), 0.0)
!       end do
!  where y(i) is the time series and lx is a power of 2
! 
!  After calling Fork with the complex array specified above, the following 
! symmetries exist:
! 
!        cx(1)        = dc value (f = 0 * df, where df = 1.0/(lx*dt))
!        cx(lx/2 + 1) = value at Nyquist (f = (lx/2+1-1)*df = 1.0/(2*dt))
!        cx(lx)       = CONJG(cx(2))
!        cx(lx-1)     = CONJG(cx(3))
!         |           =      |
!        cx(lx-i+2)   = CONJG(cx(i))
!         |           =      |
!        cx(lx/2+2)   = CONJG(cx(lx/2))
! 
! where "CONJG" is the Fortran complex conjugate intrinsic function
! 
! This symmetry MUST be preserved if modifications are made in the frequency 
! domain and another call to Fork (with a different sign for signi) is used
! to go back to the time domain.  If the symmetry is not preserved, then the
! time domain array will have nonzero imaginary components.  There is one case
! where advantage can be taken of this, and that is to find the Hilbert 
! transform and the window of a time series with only two calls to Fork (there 
! is a short note in BSSA {GET REFERENCE} discussing this trick, which amounts 
! to zeroing out the last half of the array and multiplying all but the dc and 
! Nyquist values by 2.0; in the time domain, REAL(cx(i)) and AIMAG(cx(i)) 
! contain the filtered (if a filter was applied) and Hilbert transform of the 
! filtered time series, respectively, while CABS(cx(i)) and ATAN2(AIMAG(cx(i)), 
! REAL(cx(i))) are the window and instantaneous phase of the filtered time 
! series, respectively.

! Some references:

! Farnbach, J.S. (1975). The complex envelope in seismic signal analysis, 
! BSSA 65, 951--962. 
! He states that the factor of 2 is applied for i = 2...npw2/2 (his indices 
! start at 0, I've added 1), which is different than the next reference:

! Mitra, S.K. (2001). Digital Signal Processing, McGraw-Hill, New York.
! He gives an algorithm on p. 794 (eq. 11.81), in which the factor of 2 is 
! applied from 0 frequency to just less than Nyquist.

! 
! The easiest way to ensure the proper symmetry is to zero out the
! last half of the array (as discussed above), but the following is what
! I usually use:  
! modify (filter) only half
! of the cx array:
! 
!       do i = 1, lx/2
!         cx(i) = filter(i)*cx(i)
!       end do
! 
! where "filter(i)" is a possibly complex filter function (and recall that 
! the frequency corresponding to i is f = float(i-1)*df).  After this, fill out
! the last half of the array using
!       
!       do i = lx/2+2, lx
!         cx(i) = CONJG(cx(lx+2-j))
!       end do
! 
! Note that nothing is done with the Nyquist value.  I assume (but am not sure!)
! that this value should be 0.0
! 
! Dates: xx/xx/xx - Written by Norm Brenner(?), Jon Claerbout(?)
!        12/21/00 - Replaced hardwired value of pi with pi evaluated here,
!                     and added comments regarding usage.  Also deleted
!                     dimension specification of cx(lx) and replace it with
!                     cx(*) in the type specification statement.  I also
!                     cleaned up the formatting of the subroutine.
!        08/28/01 - Added comment about variable "signi" being real, and 
!                   added "float" in equations for "sc" and "temp", although 
!                   not strictly required.
!        06/19/02 - Added some comments about computing envelopes and
!                   instantaneous frequencies
!        01/19/15 - Modernize code (get rid of go to statements)
             
      complex cx(*),carg,cexp,cw,ctemp

      pi = 4.0*atan(1.0)

      j=1
      sc=sqrt(1./real(lx))

      do i=1,lx
      
        if (i <= j) then
          ctemp=cx(j)*sc
          cx(j)=cx(i)*sc
          cx(i)=ctemp
        end if
        
        m=lx/2        
        
        DO
          if (j <= m) EXIT
          j=j-m
          m=m/2
          if (m < 1) EXIT
        END DO
        
        j = j + m
        
      end do

      l=1
      DO WHILE (l < lx)
        istep=2*l
        temp= pi * signi/real(l)

        do m=1,l
          carg=(0.,1.)*temp*(m-1)
          cw=cexp(carg)
          do i=m,lx,istep
            ctemp=cw*cx(i+l)
            cx(i+l)=cx(i)-ctemp
            cx(i)=cx(i)+ctemp
          end do
        end do

        l=istep
      END DO
 
! Previous code (as of 18 July 2010):
!      pi = 4.0*atan(1.0)
!
!      j=1
!      sc=sqrt(1./float(lx))
!
!      do i=1,lx
!        if(i.gt.j) go to 2
!        ctemp=cx(j)*sc
!        cx(j)=cx(i)*sc
!        cx(i)=ctemp
!2       m=lx/2
!3       if(j.le.m) go to 5
!        j=j-m
!        m=m/2
!        if(m.ge.1) go to 3
!5       j=j+m
!      end do
!
!      l=1
!6     istep=2*l
!      temp= pi * signi/float(l)
!
!      do m=1,l
!        carg=(0.,1.)*temp*(m-1)
!        cw=cexp(carg)
!        do i=m,lx,istep
!          ctemp=cw*cx(i+l)
!          cx(i+l)=cx(i)-ctemp
!          cx(i)=cx(i)+ctemp
!        end do
!      end do
!
!      l=istep
!      if(l.lt.lx) go to 6

      return
      end
! ----------------------------- END FORK --------------------------


! ------------------------------------------------------------- Get_NPW2
      subroutine get_npw2(npts,signnpw2,npw2)

! Find npw2 (less than npts if signnpw2 < 0)

! Dates: 12/12/00 - Written by D. Boore
!        04/04/10 - Correct error that it does not return
!                   npw2 = npts, if npts is a power of 2.
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      npw2_exp = int( alog(float(npts))/alog(2.0) )
      if (signnpw2 < 0.0) then
        npw2 = 2.0**npw2_exp
      else 
        npw2_temp = 2.0**npw2_exp
        if (npw2_temp == npts) then 
          npw2 = npts
        else
          npw2 = 2.0**(npw2_exp+1)
        end if
      end if

      return
      end
! ------------------------------------------------------------- Get_NPW2

*  ------------------- BEGIN WIND_BOX -------------------------------
      function wind_box( i, nstart, nstop, ntaper)
c
c applies cosine tapered window.
c unit amplitude assumed
c
c written by D. M. Boore
c
c latest revision: 9/26/95
      real wind_box

      wind_box = 0.0
      if ( i .lt. nstart .or. i. gt. nstop) return
      wind_box = 1.0
      if ( i .ge. nstart+ntaper .and. i .le. nstop-ntaper ) return
c
      pi = 4.0 * atan(1.0)
c
      dum1 = (nstop+nstart)/2.0
      dum2 = (nstop-nstart-ntaper)/2.0
c
      wind_box = 0.5 * (1.0 - sin( pi*
     *  ( abs(float(i)-dum1) - dum2 ) /float(ntaper) ) )
      return
      end
*  ------------------- END WIND_BOX -------------------------------

! ---------------------- BEGIN MNMAXIDX ----------------------
      subroutine mnmaxidx(a,nstrt,nstop,ninc,
     :                    amin,amax,indx_min,indx_max) 

! A rewrite of mnmax, returning the indices of the min and max values
!
! Dates: 10/05/99 - written by D. M. Boore
!        05/16/02 - Replaced "dimension a(1)" with "real a(*)"
!        11/03/12 - Replace if-then with if--then-else

      real a(*)
      amax = a( nstrt)
      indx_max = nstrt
      amin=amax         
      indx_min = nstrt
      do i=nstrt,nstop,ninc
      
        if (a(i) > amax) then
          amax=a(i)
          indx_max = i
        else if (a(i) < amin) then
          amin=a(i)
          indx_min = i
        end if
        
      end do
      
      return                  
      end                     
! ---------------------- END MNMAXIDX ----------------------


! ------------------------ BEGIN DISTAZ ----------------------
      subroutine distaz( wlongsign, alat, along, blat, blong,
     * rdeg, rkm, az, baz)
! 
! compute distances, azimuths using formulas from
! Bruce Julian.
!
! latest modification: 1/27/84
!
!        11/24/07 - Trap for colocated pair in subroutine distaz
!        05/01/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      if (alat .eq. blat .and. along .eq. blong) then
        rdeg = 0.0
        rkm = 0.0
        az = 0.0
        baz = 0.0
        return
      end if

      pi = 4.0 * atan( 1. )
      dtor = pi/ 180.
!
! convert from degrees to radians and correct sign of
! longitude so that east longitude is positive.
!
      alatr = dtor * alat
      alongr = -dtor * along * wlongsign
      blatr = dtor * blat
      blongr = -dtor * blong * wlongsign
!
! compute geocentric latitudes.
!
      alatr = atan( 0.993305 * tan( alatr ) )
      blatr = atan( 0.993305 * tan( blatr ) )
!
! compute latitude dependent quantities
!
      ca = cos( alatr )
      cb = cos( blatr )
      sa = sin( alatr )
      sb = sin( blatr )
!
! now compute other quantities
!
      a = cb * sin( blongr - alongr )
      b = ca * sb - sa * cb * cos( blongr - alongr )
      cd = ca * cb * cos( blongr - alongr ) + sa * sb
      sd = sqrt( a*a + b*b )
!
! compute distances
!
      rdeg = atan2( sd, cd )/ dtor
      rkm = 111.19 * rdeg
!
! compute azimuth (from a to b) and make it positive.
!
      az = atan2( a, b )/ dtor
      if ( az .lt. 0.0 ) az = az + 360.0
!
! compute back azimuth (from b to a) and make it positive.
!
      a = ca * sin( alongr - blongr )
      b = cb * sa - sb * ca * cos( alongr - blongr )
      baz = atan2( a, b)/ dtor
      if ( baz .lt. 0.0 ) baz = baz + 360.0
!
      return
      end
! ------------------------ END DISTAZ ----------------------

! --------------------- BEGIN UPSTR ----------------------------------
      Subroutine UPSTR ( text )
! Converts character string in TEXT to uppercase
! Dates: 03/12/96 - Written by Larry Baker
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

!
      Implicit   None
!
      Character  text*(*)
!
      Integer    j
      Character  ch
!
      Do 1000 j = 1,LEN(text)
         ch = text(j:j)
         If ( LGE(ch,'a') .and. LLE(ch,'z') ) Then
            text(j:j) = CHAR ( ICHAR(ch) - ICHAR('a') + ICHAR('A') )
         End If
 1000    Continue
!
      Return
      End
! --------------------- END UPSTR ----------------------------------
! --------------------------- BEGIN TRIM_C -----------------------
      subroutine trim_c(cstr, nchar)

! strips leading and trailing blanks from cstr, returning the
! result in cstr, which is now nchar characters in length

! Strip off tabs also.

! Here is a sample use in constructing a column header, filled out with 
! periods:

!* Read idtag:
!        idtag = ' '
!        read(nu_in, '(1x,a)') idtag
!        call trim_c(idtag, nc_id)
!* Set up the column headings:
!        colhead = ' '
!        colhead = idtag(1:nc_id)//'......' ! nc_id + 6 > length of colhead

! Dates: 12/23/97 - written by D. Boore
!        12/08/00 - pad with trailing blanks.  Otherwise some comparisons
!                   of the trimmed character string with other strings
!                   can be in error because the original characters are left
!                   behind after shifting.  For example, here is a string
!                   before and after shifting, using the old version:
!                      col:12345
!                           MTWH  before
!                          MTWHH  after (but nc = 4).
!        03/21/01 - Check for a zero length input string
!        11/09/01 - Change check for zero length string to check for all blanks
!        10/19/09 - Strip off tabs
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)
!        02/14/18 - Use "len_trim" to check for cstr being all blank

      character cstr*(*)

      nend = len_trim(cstr)

      if(nend == 0) then
        nchar = 0
        return
      end if

! Replace tabs with blanks:

      do i = 1, nend
        if(ichar(cstr(i:i)) == 9) then
           cstr(i:i) = ' '
        end if
      end do



!      if(nend == 0) then
!        nchar = 0
!        return
!      end if

      do i = nend, 1, -1
        if (cstr(i:i) /= ' ') then
           nchar2 = i
           goto 10
        end if
      end do

10    continue

      do j = 1, nchar2
        if (cstr(j:j) /= ' ') then
          nchar1 = j
          goto 20
        end if
      end do

20    continue
   
      nchar = nchar2 - nchar1 + 1
      cstr(1:nchar) = cstr(nchar1: nchar2)
      if (nchar < nend) then
        do i = nchar+1, nend
          cstr(i:i) = ' '
        end do
      end if

      return
      end
! --------------------------- END TRIM_C -----------------------! ---------------------- BEGIN SKIP -------------------
      subroutine SKIP(lunit, nlines)
        if (nlines .lt. 1) then
          return
        else
          do i = 1, nlines
             read(lunit, *)
          end do
          return
        end if
      end
! ---------------------- END SKIP -------------------
! ------------------------------------------------------------------ skipcmnt
      subroutine skipcmnt(nu, comment, ncomments)

! Skip text comments in the file attached to unit nu, but save skipped 
! comments in character array comment.  Skip at least one line, and more as 
! long as the lines are preceded by "|" or "!".

! Dates: 04/16/01 - Written by D. Boore
!        12/07/01 - Added check for eof
!        11/04/03 - Use trim_c to trim possible leading blank
!        02/03/07 - Initialize comments to blank
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      character comment(*)*(*), buf*80

      ncomments = 0
100   buf = ' '
      read (nu,'(a)',end=999) buf
      call trim_c(buf,nc_buf)
      if (buf(1:1) .eq.'!' .or. buf(1:1) .eq.'|' .or. 
     :                     ncomments + 1 .eq. 1) then
        ncomments = ncomments + 1
        comment(ncomments) = ' '
        comment(ncomments) = buf(1:nc_buf)
        goto 100
      else 
        backspace nu
      end if

999   continue
 
      return
      end
! ------------------------------------------------------------------ skipcmnt

! --------------------------- BEGIN GET_LUN ----------------
      subroutine get_lun(lun)

! Finds a logical unit number not in use; returns
! -1 if it cannot find one.

! Dates -- 05/19/98 - Written by D. Boore, following
!                     Larry Baker's suggestion
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      logical isopen
      do i = 99,10,-1
        inquire (unit=i, opened=isopen)
        if(.not.isopen) then
          lun = i
          return
        end if
      end do
      lun = -1

      return
      end
! --------------------------- END GET_LUN ----------------
     

!------------------   Begin SMCRead   ----------------------------------
      subroutine SMCRead(f_in,  
     :                   tskip, tlength,
     :                   y, char_head, int_head, real_head, comments)

!  If tskip = 0.0, no data are skipped
!  If tlength = 0.0, only headers are read
!  If tlength < 0.0, read all data, less tskip
!  Otherwise tlength of data is read, after skipping tskip, unless 
!  tskip + tlength exceeds the length of the input time series, in which
!  case tlength is adjusted so that all data, less tskip, are read.  Thus
!  setting tlength to a large number (or setting it to a negative number)
!  guarantees that all data (minus tskip) are read.

! The program requires get_lun.for

! The output arrays char_head, int_head, real_head should be dimensioned
! as follows in the calling program:

!      character char_head(11)*80, comments(500)*80 ! "80" is the critical parameter
!      integer int_head(48)
!      real real_head(50)

!  Dates: 07/17/97 - Modified order of input parameters, to put input
!                    first (unit, file, tskip), followed by output.
!         12/18/97 - Read tlength
!         05/19/98 - Determine unit from within program; remove "unit"
!                    from argument list
!         03/09/99 - Change logic to determine amount of record to
!                    read and allow reading of all record, minus tskip,
!                    by setting tlength < 0.0.
!         05/18/99 - Deleted npts, sps from input parameter list.  Added 
!                    character array with comments string (allow for 
!                    50 comments).
!         12/16/99 - Renamed from ReadSMC
!         05/10/00 - Replaced
!                        tlength = float(npts_out/sps)
!                    with
!                        tlength = float(npts_out)/sps
!                    (the error and correction was pointed out by Wang Guoquan
!                    in email on 05/06/2000)
!         01/04/01 - Check for unevenly spaced data (sps = rnull) and
!                    return 0.0 in y
!         02/07/01 - If tlength = 0, only read headers
!         12/14/01 - Fixed logic for case when tlength < 0 and tskip > 0.
!                    Also, program used to reset tlength, but it does not do
!                    that now.  The user can compute tlength as 
!                    float(int_head(17))/real_head(2)
!         01/01/05 - Use "int" in calculation of npts (do not use
!                    implicit conversion from real to integer)
!         10/15/07 - Use higher precision input format if int_head(47) = 8
!         04/04/10 - Add call to trim_c, and use nc_f_in.
!         10/20/15 - Changed format of high precision input.
!                  - This was not necessary, as the old format statement can read
!                    either e14.7 or e14.7e1 (the new format being used in smcwrite for
!                    high precision).
!         12/30/15 - Replaced "int" with "nint" when computing nskip and npts2read
!                  - A few modernizations of syntax.
!         05/04/17 - replaced "float" with the generic function "real"

      real real_head(*)
      integer int_head(*)
      character*80 char_head(*), comments(*)
      character f_in*(*) 
      real y(*)

      
      
      call get_lun(nu)
      call trim_c(f_in, nc_f_in)
      open(unit= nu, file=f_in(1:nc_f_in), status='unknown')

      do i = 1, 11
        read(nu, '(a)') char_head(i)
      end do

      read(nu, '(8I10)') (int_head(i), i=1, 48)

      read(nu, '(5e15.7)') (real_head(i), i = 1, 50)

      sps = real_head(2)

      do i = 1, int_head(16)
        read(nu,'(a)') comments(i)
      end do

      if(real_head(2) == real_head(1)) then   ! unevenly sampled data

         int_head(17) = 2
         do i = 1, int_head(17)
           y(i) = 0.0
         end do

      else

! Skip into the trace:

        nskip = nint(tskip * sps)
!        nskip = int(tskip * sps)

! How much record to read?

        npts_in = int_head(17)
        npts2read = nint(tlength * sps)
!        npts2read = int(tlength * sps)

        if (tlength < 0.0) then  
          npts_out = npts_in - nskip
        else if (nskip + npts2read <= npts_in) then
          npts_out = npts2read
        else
          npts_out = npts_in - nskip
!          tlength = real(npts_out)/sps
        end if

        int_head(17) = npts_out
      
        if (tlength == 0.0) then
          return    ! only read headers
        else
          if (int_head(47) == 8) then
            read(nu, '(5(e14.7))') (y(i), i = 1, npts_out + nskip)
!            read(nu, '(5(e14.7e1))') (y(i), i = 1, npts_out + nskip)
          else
            read(nu, '(8(1pe10.4e1))') (y(i), i = 1, npts_out + nskip)
          end if
          do i = 1, npts_out
            y(i) = y( i + nskip)
          end do
        end if

      end if

      close(unit=nu)

      return
      end
!------------------   End SMCRead   ----------------------------------





!------------------   Begin SMCWrite  ----------------------------------
      subroutine SMCWrite(f_out, precision,
     :                    y, char_head, itype, 
     :                    int_head, real_head, comments)

!  Reformats a time series 
!  into SMC format (the format used on the CD-ROM)

! If itype < 0, the first text header is given by char_head(1) and
!               not set to one of the standard headers
!    itype = 0 for 'UNKNOWN'
!    itype = 1 for 'UNCORRECTED ACCELEROGRAM'
!    itype = 2 for 'CORRECTED ACCELEROGRAM'
!    itype = 3 for 'VELOCITY'
!    itype = 4 for 'DISPLACEMENT'
!    itype = 5 for 'RESPONSE SPECTRA'

! The program requires:
!        get_lun.for
!        mnmaxidx.for

! The arrays char_head, int_head, real_head must be dimensioned
! by the calling program as follows:
!
!      character char_head(11)*80, comments(N)*80 ! see "N" below
!      integer int_head(48)
!      real real_head(50)

! Note that 
!      int_head(17) = npts
!      real_head(2) = sps
!      int_head(16) = ncomments (dimension array "comments" accordingly
!                                so that N is a number > = ncomments)
!      I have found that if no comments are included, BAP cannot process
!      the resulting file.  For this reason, be sure that at least one comment
!      is included (ncomments = 1, comments(1) = '|').
!
! default values for int_head = -32768
!                    real_head = 1.7e+38

! The minimum required input for char_head is to set each of the 11 variables
! to "*" as in:
!  do i = 1, 11
!    char_head(i) = '*'
!  end do


! Dates: 05/13/99 - revised by D. Boore
!        05/18/99 - continued, adding the comments string array
!        12/15/99 - Renamed from WriteSMC to SMCWrite, and added
!                   itype to argument list and find and write peak
!                   accelerations (itype < 0 to echo input char_head(1)).
!        01/29/00 - Added comments regarding dimensioning of input variables
!        02/18/00 - Substitute 0.0 if abs(y) < 1.0e-09 (otherwise, the field
!                   is stored as ********, because an exponent of -10 is
!                   larger than allowed by the format statement).
!        02/25/00 - Changed output format of peak motion in text headers from
!                   1pe10.3 to 1pe9.3e1
!        02/27/01 - Set char_head(7)(42:60) to blanks before filling with
!                   peak motion
!        08/22/01 - Added comment about minimum requirement for char_head
!        10/15/07 - Use high precision format if precision=HIGH
!        04/24/09 - Changed format of read header output from 5e15.7 to 1p5e15.7
!        04/04/10 - Add call to trim_c, and use nc_f_out.
!        02/02/12 - Compute r_ep if eq and sta coords /= null
!        06/24/13 - Replaced ".eq." with "==", etc.
!                 - Replaced 1pex.x with esx.x
!                 - In writing real_head, revert to the format 5e15.7 rather than 5es15.7,
!                   as the latter might imply more precision than available in the
!                   single precision numbers.
!        10/20/15 - Change high precision output format to include blanks between
!                   numbers (as suggested by Chris Stephens)
!        10/24/15 - For both output precision options substitute 0.0 if abs(y) < 1.0e-09 (otherwise, the field
!                   is stored as ********, because an exponent of -10 is
!                   larger than allowed by the format statement).
!        04/17/17 - Use es15.5 rather than e15.7 for real_head(2). Using es15.6 gives real_head(2)=9.999999E+02 
!                   rather than 1.000000E+03 when dt = 0.001.  It could be that all real_head values should be written
!                   using less precision, but for now I will only change real_head(2), because
!                   the precision problem is most obvious for that real_head.

      real real_head(*)
      integer int_head(*)
      character*80 char_head(*), comments(*)
      character f_out*(*), chead1(6)*26, precision*(*)
      real y(*)
      data chead1/
     : '0 UNKNOWN                 ', '1 UNCORRECTED ACCELEROGRAM',
     : '2 CORRECTED ACCELEROGRAM  ', '3 VELOCITY                ',
     : '4 DISPLACEMENT            ', '5 RESPONSE SPECTRA        '/

! Open output file:
      call get_lun(nu_out)
      call trim_c(f_out, nc_f_out)
      open(unit=nu_out, file=f_out(1:nc_f_out), status='unknown')

! Write name to screen:
      print *, 
     :  ' nu_out, Output file name = ', nu_out, ', ', f_out(1:nc_f_out)

! Get peak motions:
      
      call mnmaxidx(y, 1, int_head(17), 1,
     :             ymin, ymax, indx_min, indx_max) 
      peak_mtn = ymax
     
      if (abs(ymin) > abs(ymax)) peak_mtn = ymin
      
! Compute r_ep if eq, sta coords /= null:  NOTE: Need to include distaz in the calling program

      rnull = real_head(1)
      if (real_head(3) /= rnull .and. real_head(3) /= rnull) then  ! eq coords not null
        if (real_head(11) /= rnull .and. real_head(12) /= rnull) then  ! sta coords not null
          eqlat = real_head(3)
          eqlong = real_head(4)
          stalat = real_head(11)
          stalong = real_head(12)
          call distaz( -1.0, eqlat,eqlong,stalat,stalong,
     :    rdeg, rkm, az, baz)
          real_head(17) = rkm
        end if
      end if
          

! Write headers:

! First write the 11 lines of comments:

      if (itype < 0) then
        write(nu_out,'(a)') char_head(1)
      else
        write(nu_out,'(a)') chead1(itype+1)
      end if

      char_head(7)(34:41) = 'pk mtn ='    ! makes more sense than 'pk acc ='
      char_head(7)(42:60) = ' '
      write(char_head(7)(42:50),'(1pe9.3e1)') peak_mtn

      do i = 2, 11
        write(nu_out,'(a)') char_head(i)
      end do
          
      if (precision(1:4) == 'HIGH') then
        int_head(47) = 8
      else
        int_head(47) = int_head(1) ! null value
      end if
      
      write(nu_out, '(8I10)') (int_head(i), i=1, 48)

      dt = 1.0/real_head(2)
      real_head(29) = float(indx_max - 1) * dt
      real_head(30) = ymax
      real_head(31) = float(indx_min - 1) * dt
      real_head(32) = ymin

!      write(nu_out, '(5es15.7)') (real_head(i), i = 1, 50)
!DEBUG
!      write(nu_out, '(a, 1x,e15.7, 1x,f7.2, 1x,es12.5)') 
!     :   ' In SMCWRITE: real_head(2), dt = ', 
!     :     real_head(2), real_head(2), dt
!DEBUG
!DEBUG
!      write(*, '(a, 1x,e15.7, 1x,f7.2, 1x,es12.5)') 
!     :   ' In SMCWRITE: real_head(2), dt = ', 
!     :     dble(real_head(2)), real_head(2), dt
!DEBUG
!      write(nu_out, '(5e15.7)') (real_head(i), i = 1, 50)

      write(nu_out, '(es15.6,es15.5,3es15.6)') (real_head(i), i = 1, 5)
      write(nu_out, '(5es15.6)') (real_head(i), i = 6, 50)
!      write(nu_out, '(5es15.6)') (real_head(i), i = 1, 50)


      if (int_head(16) > 0) then
        do i = 1, int_head(16)
          write(nu_out,'(a)') comments(i)
        end do
      end if
          
        do i = 1, int_head(17)
          if (abs(y(i)) < 1.0e-09) y(i) = 0.0
        end do

      if (precision(1:4) == 'HIGH') then
        write(nu_out, '(5(es14.7e1))') (y(i), i = 1, int_head(17))
      else
!        do i = 1, int_head(17)
!          if (abs(y(i)) < 1.0e-09) y(i) = 0.0
!        end do
        write(nu_out, '(8(es10.4e1))') (y(i), i = 1, int_head(17))
      end if
      

! That should be it; close the output file.

      close(unit=nu_out)

      return
      end
!------------------   End SMCWrite   ----------------------------------


! ------------------------------------------------------------- SMC_Npts
      subroutine smc_npts(f_smc, npts, sps)

* Open smc file and obtain length of time series:

* Dates: 12/07/00 - Written by D. Boore
*        07/17/03 - Read all of int_head and real_head, and add sps to output
!        04/04/10 - Add trim_c, and use nc_f_smc.

      integer int_head(48)
      real real_head(50)
      character f_smc*(*)

      call get_lun(nu_smc)
      call trim_c(f_smc,nc_f_smc)
      open(unit=nu_smc,file=f_smc(1:nc_f_smc),status='unknown')
      call skip(nu_smc,11)

      read(nu_smc, '(8I10)') (int_head(i), i=1, 48)
      npts = int_head(17)

      read(nu_smc, '(5e15.7)') (real_head(i), i = 1, 50)
      sps = real_head(2)

      close(nu_smc)        

      return
      end
! ------------------------------------------------------------- SMC_Npts  

! ---------------------- BEGIN IMNMAX ----------------------
      subroutine imnmax(ia,nstrt,nstop,ninc,imin,imax) 
c
* Compute min, max for an integer array

c author: D. M. Boore
c last change: 7/17/97 - Written, based on mnmax.for
c
      integer ia(*)
      imax = ia( nstrt)
      imin=imax         
      do 10 i=nstrt,nstop,ninc
      if(ia(i)-imax) 15,15,20  
20    imax=ia(i)               
      go to 10                
15    if(ia(i)-imin) 25,10,10  
25    imin=ia(i)               
10    continue                
      return                  
      end                     
! ---------------------- END IMNMAX ----------------------

! -------------------------------------------------------------------- FILTER
      subroutine filter(a, npts, dt, flc, nroll_lc, icaus_lc,
     :                  fhc, nroll_hc, icaus_hc, icaus_bn, d4notch, 
     :                  filter_type, nc_filter_type)

! Butterworth filter time series stored in 'a', according to the following:
! if fhc = 0 = flc       : no filtering
! if fhc = 0             : low-cut filter
! if flc = 0             : high-cut filter
! if neither flc, fhc = 0: band-pass filter
! if flc = fhc /= 0, a notch filter, where the width parameter d is given by real(nslope_lc)*10**(-real(nslope_hc))

! "bn" in "icaus_bn" means "Band" "Notch"; icaus_bn = 1 for a band or notch filter
! only if icaus_lc = icaus_hc = 1.

!
! Dates: 09/11/00 - Written by D. Boore
!        02/04/01 - Added separate nroll for lc and hc filters; for
!                   the bandpass filter the max of nroll_lc and nroll_hc 
!                   is used.  In the future, the bandpass subroutine may be 
!                   changed to allow for separate rolloffs at low and high
!                   frequencies.
!        02/20/10 - Added notch filter, and the d parameter for this filter
!        07/11/17 - Allow the low- and high-cut filters to be either causal or acausal, independently
!                   (previously they had to both be one or the other).
!        07/13/17 - Change input parameters (determine filter type in calling program)
      
      real a(*)
      character filter_type*(*)

      if (filter_type(1:nc_filter_type) == 'NF') then
        return
      else if (filter_type(1:nc_filter_type) == 'LC') then
        call locut(a, npts, flc, dt, nroll_lc, icaus_lc)
      else if (filter_type(1:nc_filter_type) == 'HC') then
        call hicut(a, npts, fhc, dt, nroll_hc, icaus_hc)
      else if (filter_type(1:nc_filter_type) == 'LCHC') then
        call locut(a, npts, flc, dt, nroll_lc, icaus_lc)
        call hicut(a, npts, fhc, dt, nroll_hc, icaus_hc)
      else if (filter_type(1:nc_filter_type) == 'BAND') then
        nroll_bp = max0(nroll_lc, nroll_hc)
        call band(a, npts, flc, fhc, dt, nroll_bp, icaus_bn)  
      else if (filter_type(1:nc_filter_type) == 'NOTCH') then
        call notch(a, npts, flc, dt, d4notch, icaus_bn)
      else
        write(*,'(a)') ' In filter, unknown filter type; quitting!!! '
        stop
      end if

      return
      end
! -------------------------------------------------------------------- FILTER    

!* ---------------------------------------------------------------------- LOCUT
      subroutine locut(s,nd,fcut,delt,nroll,icaus)

c  Butterworth locut filter order 2*nroll (nroll<=8) (see Kanasewich, 
c    Time Sequence Analysis in Geophysics, Third Edition, 
c    University of Alberta Press, 1981)
c  written by W. B. Joyner 01/07/97
c  causal if icaus.eq.1 - zero phase shift otherwise
c  s(j) input = the time series to be filtered - output = the 
c    filtered series
c  dimension of s(j) must be at least as large as 
c    nd+3.0*float(nroll)/(fcut*delt)
c  nd = the number of points in the time series
c  fcut = the cutoff frequency
c  delt = the timestep

* There is a fundamental difference in the shapes of a causal and acausal 
* filter.

* The response for a causal filter is given by eq. 15.8-6 in Kanasewich:
*    Y = sqrt((f/fcut)**(2*n)/(1+(f/fcut)**(2*n))), where n = 2*nroll
* The acausal filter is obtained by applying the causal filter twice.
* The consequence is that the response of the acausal filter is
*    Y =      (f/fcut)**(2*n)/(1+(f/fcut)**(2*n)),  where n = 2*nroll
* Note that for the same nroll as input, for f/fcut << 1
* the causal filter goes as
*    Y -> (f/fcut)**(2*nroll)
* whereas the acausal filter falls off more rapidly:
*    Y -> (f/fcut)**(4*nroll)
* Also note the difference when f = fcut:
*   causal:  Y = 1/sqrt(2)
*  acausal:  Y = 1/2
* This is true no matter what value is chosen for nroll.  Because of this,
* it is not possible to make the response of the causal and acausal filters the
* same.  In particular, caution should be used if nroll is chosen so as to
* make the low-frequency asymptotes the same.  This requires nroll for the 
* acausal filter to be half that of the causal filter.  The problem is that
* the response for frequencies higher than fcut will be reduced for the
* acausal as compared to the causal filter.  


* Dates: xx/xx/xx - Written by Bill Joyner
*        09/30/97 - double precision statements added so that the output
*                   series has sufficient precision for double integration.
*        12/17/99 - D. Boore added check for fcut = 0.0, in which case
*                   no filter is applied.  He also cleaned up the
*                   appearance of the code (indented statements in
*                   loops, etc.)
*        02/04/00 - Changed "n" to "nroll" to eliminate confusion with
*                   Kanesewich, who uses "n" as the order (=2*nroll)
*        09/12/00 - Made trig functions double precision
*        11/08/00 - Increased dimension of s from 50000 to 100000
*        02/14/01 - Modified/corrected real numbers function calls to 
*                   double precision - cds
*        02/15/01 - Added response of acausal filter in the comments
*        02/22/01 - Removed dimension of s (it is up to the user to specify
*                   it properly)
*        02/22/01 - Added more discussion about the shapes of the causal
*                   and acausal filters.
*        07/20/01 - No filter of fcut .le. 0.0
!        08/01/12 - Added "dble" to b1(k)= statement and changed "2.0d0" to "2d0"
!                   (before doing that, part of the statement was highlighted in
!                   red when viewed in Lahey's ED4W, although I don't know why).

      real s(*)
      real*8 fact(8),b1(8),b2(8)
      real*8 pi,w1,xp,yp,x1,x2,y1,y2


      if (fcut .le. 0.0) return       ! Added by DMB

      pi=4.0d0*datan(1.0d0)
      w1=2.0d0*pi*fcut
      w1=2.0d0*dtan(w1*delt/2.0d0)/delt

      do k=1,nroll
        fact(k)=1.0d0/(1.0d0+dsin(pi*dfloat(2*k-1)/dfloat(4*nroll))
     *   *w1*delt+
     *   (w1*delt/2.0d0)**2)
        b1(k)=( (w1*dble(delt))**2/2d0-2d0 )*fact(k)
        b2(k)=(1.0d0-dsin(pi*dfloat(2*k-1)/dfloat(4*nroll))*w1*delt+
     *   (w1*delt/2.0d0)**2)*fact(k)
      end do

      np2=nd

      if(icaus.ne.1) then
        npad=3.0*float(nroll)/(fcut*delt)
        np1=nd+1
        np2=nd+npad
        do j=np1,np2
          s(j)=0.0
        end do
      end if

      do k=1,nroll
        x1=0.0d0
        x2=0.0d0
        y1=0.0d0
        y2=0.0d0
        do j=1,np2
          xp=s(j)
          yp=fact(k)*(xp-2.0d0*x1+x2)-b1(k)*y1-b2(k)*y2
          s(j)=yp
          y2=y1
          y1=yp
          x2=x1
          x1=xp
        end do
      end do

      if(icaus.ne.1) then
        do k=1,nroll
          x1=0.0d0
          x2=0.0d0
          y1=0.0d0
          y2=0.0d0
          do j=1,np2
            xp=s(np2-j+1)
            yp=fact(k)*(xp-2.0d0*x1+x2)-b1(k)*y1-b2(k)*y2
            s(np2-j+1)=yp
            y2=y1
            y1=yp
            x2=x1
            x1=xp
          end do
        end do
      end if

      return
      end
!* ---------------------------------------------------------------------- LOCUT

!* --------------------------------------------------------------------- HICUT
      subroutine hicut(s,nd,fcut,delt,n,icaus)

!  Butterworth hicut filter order 2*n (n<=8) (see Kanasewich, 
!    Time Sequence Analysis in Geophysics, Third Edition, 
!    University of Alberta Press,1981)
!  written by W. B. Joyner 01/07/97
!  causal if icaus.eq.1 - zero phase shift otherwise
!  s(j) input = the time series to be filtered - output = the 
!    filtered series
!  dimension of s(j) must be at least as large as
!    nd+3.0*float(n)/(fcut*delt)
!  nd = the number of points in the time series
!  fcut = the cutoff frequency
!  delt = the timestep
! Dates: xx/xx/xx - Written by Bill Joyner
!        01/07/00 - D. Boore added check for fcut = 0.0d0, in which case
!                   no filter is applied.  He also cleaned up the
!                   appearance of the code (indented statements in
!                   loops, etc.)
!        09/12/00 - double precision statements added so that the output
!                   series has sufficient precision for double integration.
!        11/08/00 - Increased dimension of s from 50000 to 100000
!        02/14/01 - Modified/corrected real numbers function calls to 
!                   double precision - cds
!        02/22/01 - Removed dimension of s (it is up to the user to specify
!                   it properly)
!        07/20/01 - No filter of fcut .le. 0.0
!        11/18/12 - Increased array dimensions for that n<=16

      real s(*)
      real*8 fact(16),b1(16),b2(16)
!      real*8 fact(8),b1(8),b2(8)
      real*8 pi,w1,xp,yp,x1,x2,y1,y2

      if (fcut <= 0.0) return       ! Added by DMB

      pi=4.0d0*datan(1.0d0)
      w1=2.0d0*pi*fcut
      w1=2.0d0*dtan(w1*delt/2.0d0)/delt

      do k = 1, n
        fact(k)=1.0d0/((2.0d0/(w1*delt))**2+
     :   4.0d0*dsin(pi*dfloat(2*k-1)/dfloat(4*n))/(w1*delt)+1.0d0)
        b1(k)=2.0d0*(1.0d0-(2.0d0/(w1*delt))**2)*fact(k)
        b2(k)=((2.0d0/(w1*delt))**2-
     :   4.0d0*dsin(pi*dfloat(2*k-1)/dfloat(4*n))/(w1*delt)+1.0d0)
     :   *fact(k)
      end do

      np2=nd

      if (icaus /= 1) then
        npad=3.0d0*dfloat(n)/(fcut*delt)
        np1=nd+1
        np2=nd+npad
        do j = np1, np2
          s(j)=0.0
        end do
      end if

      do k = 1, n
        x1=0.0d0
        x2=0.0d0
        y1=0.0d0
        y2=0.0d0
        do j = 1, np2
          xp=s(j)
          yp=fact(k)*(xp+2.0d0*x1+x2)-b1(k)*y1-b2(k)*y2
          s(j)=yp
          y2=y1
          y1=yp
          x2=x1
          x1=xp
        end do
      end do

      if(icaus /= 1) then
        do k = 1, n
          x1=0.0d0
          x2=0.0d0
          y1=0.0d0
          y2=0.0d0
          do j = 1, np2
            xp=s(np2-j+1)
            yp=fact(k)*(xp+2.0d0*x1+x2)-b1(k)*y1-b2(k)*y2
            s(np2-j+1)=yp
            y2=y1
            y1=yp
            x2=x1
            x1=xp
          end do
        end do
      end if

      return
      end
!* --------------------------------------------------------------------- HICUT


!* ---------------------------------------------------------------------- BAND
      subroutine band(s,nd,f1,f2,delt,nroll,icaus)
c  Butterworth bandpass filter order 2*nroll (nroll<=8) (see Kanasewich, 
c    Time Sequence Analysis in Geophysics, Third Edition, 
c    University of Alberta Press, 1981)
c  written by W.B. Joyner 01/07/97
c  causal if icaus.eq.1 - zero phase shift otherwise
c  s(j) input = the time series to be filtered - output = the 
c    filtered series
c  dimension of s(j) must be as least as large as the larger of 
c    the following:
c    nd+3.0*float(nroll)/(f1*delt)
c    nd+6.0*float(nroll)/((f2-f1)*delt)
c  nd = the number of points in the time series
c  f1, f2 = the cutoff frequencies
c  delt = the timestep

* Dates: xx/xx/xx - Written by Bill Joyner
*        09/12/00 - Changed "n" to "nroll" to eliminate confusion with
*                   Kanesewich, who uses "n" as the order (=2*nroll), and
*                   cleaned up appearance of code by using spaces, indents, etc.
*        09/12/00 - double precision statements added so that the output
*                   series has sufficient precision for double integration.
*        11/08/00 - Increased dimension of s from 50000 to 100000
*        02/14/01 - Modified/corrected real numbers function calls to 
*                   double precision - cds
*        02/22/01 - Removed dimension of s (it is up to the user to specify
*                   it properly)
!        06/26/14 - Adopted some minor suggested changes by Chris Stephens

      real s(*)
      real*8 fact(16),b1(16),b2(16)
      real*8 pi,w1,w2,xp,yp,x1,x2,y1,y2
      real*8 pre, pim, argre, argim, rho, theta, sjre, sjim
      real*8 bj, cj, con

      if(f1.eq.0..or.f1.ge.f2) return  ! Changed "f1.eq.f2" to "f1.ge.f2" on 06/26/14

      pi=4.0d0*datan(1.0d0)
      w1=2.0d0*pi*f1
      w1=2.0d0*dtan(w1*delt/2.0d0)/delt
      w2=2.0d0*pi*f2
      w2=2.0d0*dtan(w2*delt/2.0d0)/delt

      do k=1,nroll
        pre=-dsin(pi*dfloat(2*k-1)/dfloat(4*nroll))
        pim=dcos(pi*dfloat(2*k-1)/dfloat(4*nroll))
        argre=(pre**2-pim**2)*(w2-w1)**2/4.0d0-w1*w2
        argim=2.0d0*pre*pim*(w2-w1)**2/4.0d0
        rho=(argre**2+argim**2)**(1.0d0/4.0d0)
        theta=pi+datan2(argim,argre)/2.0d0
        do i=1,2
          sjre=pre*(w2-w1)/2.0d0+(-1)**i*rho*(-dsin(theta-pi/2.0d0))
          sjim=pim*(w2-w1)/2.0d0+(-1)**i*rho*dcos(theta-pi/2.0d0)
          bj=-2.0d0*sjre
          cj=sjre**2+sjim**2
          con=1.0d0/(2.0d0/delt+bj+cj*delt/2.0d0)
          fact(2*k+i-2)=(w2-w1)*con
          b1(2*k+i-2)=(cj*delt-4.0d0/delt)*con
          b2(2*k+i-2)=(2.0d0/delt-bj+cj*delt/2.0d0)*con
        end do
      end do

      np2=nd

      if(icaus.ne.1) then
        npad=3.0*float(nroll)/(f1*delt)
        if( npad .lt. 6.0*float(nroll)/((f2-f1)*delt) ) then
          npad=6.0*float(nroll)/((f2-f1)*delt)
        end if
        np1=nd+1
        np2=nd+npad
        do j=np1,np2
          s(j)=0.0
        end do
      end if

      do k=1,2*nroll
        x1=0.0d0
        x2=0.0d0
        y1=0.0d0
        y2=0.0d0
        do j=1,np2
          xp=s(j)
          yp=fact(k)*(xp-x2)-b1(k)*y1-b2(k)*y2
          s(j)=yp
          y2=y1
          y1=yp
          x2=x1
          x1=xp
        end do
      end do

      if(icaus.ne.1) then
        do k=1,2*nroll
          x1=0.0d0
          x2=0.0d0
          y1=0.0d0
          y2=0.0d0
          do j=1,np2
            xp=s(np2-j+1)
            yp=fact(k)*(xp-x2)-b1(k)*y1-b2(k)*y2
            s(np2-j+1)=yp
            y2=y1
            y1=yp
            x2=x1
            x1=xp
          end do
        end do
      end if

      return

      end
!* ---------------------------------------------------------------------- BAND


!* ------------------------ BEGIN NOTCH ------------------------
      subroutine notch(s,nd,fr,delt,d,icaus)
c  notch filter (see Kanasewich, Time Sequence Analysis in 
c    Geophysics, Third Edition, University of Alberta Press, 
c    1981; Shanks, Feb. 1967 Geophysics)
c  written by W. B. Joyner 01/07/97
c  causal if icaus.eq.1 - zero phase shift otherwise
c  s(j) input = the time series to be filtered - output = the 
c    filtered series
c  dimension of s(j) must be at least as large as
c    nd+3.0*pi/d
c  nd = the number of points in the time series
c  fr = the reject frequency
c  delt = the timestep
c  d = parameter << 1 - the smaller d the narrower the notch - 
c    the value 0.01 is suggested for first trial
* Dates: 01/07/97 - Written by W. Joyner
*        07/20/01 - Option to return (no filtering) if fr < 0.0
!        03/20/10 - Tidied up by D. Boore

      real s(*)
      pi=4.0*atan(1.0)
      
      IF (fr < 0.0) THEN
        return
      END IF
      
      r = 1.0+d
      w = pi*fr*delt*2.0
      cosw = cos(w)
      fact = (1.0+2.0*cosw/r+1.0/r**2)/(2.0*(1.0+cosw))
      a1 = -2.0*cosw
      b1 = -2.0*cosw/r
      b2 = 1.0/r**2
      np2 = nd
      
      IF (icaus /= 1) THEN
      
        npad = 3.0*pi/d
        np1 = nd+1
        np2 = nd+npad
        
        DO j = np1, np2
          s(j) = 0.0
        END DO
        
      END IF
      
      x1 =0.0
      x2 =0.0
      y1 =0.0
      y2 =0.0
      
      DO j = 1, np2
      
        xp  = s(j)
        yp  = fact*(xp+a1*x1+x2)-b1*y1-b2*y2
        s(j) = yp
        y2 = y1
        y1 = yp
        x2 = x1
        x1 = xp
        
      END DO
      
      IF (icaus /= 1) THEN
      
        x1 = 0.0
        x2 = 0.0
        y1 = 0.0
        y2 = 0.0
        
        DO j = 1, np2 
        
          xp = s(np2-j+1)
          yp = fact*(xp+a1*x1+x2)-b1*y1-b2*y2
          s(np2-j+1) = yp
          y2 = y1
          y1 = yp
          x2 = x1
          x1 = xp
          
        END DO
        
      END IF
      
      RETURN
      
      END
!* ------------------------ END NOTCH ------------------------


!----------------- BEGIN Trapezoidal_Integration -----------------------------
      subroutine Trapezoidal_Integration(y, npts, dt, yint_0, yint)


* Integrate a time series y with initial condition yint_0

 
* Dates: 02/07/11 - Written by D.M. Boore, patterned after acc2vd.for
      real y(*), yint(*) 
      double precision cumv, y1, y2, v1, v2,
     : ddt, ddt_2 

 
* Integrate

      ddt     = dble(dt)
      ddt_2   = 0.5d0 * dble(dt)
 
      cumv = 0.0d0
 
      v1 = 0.0d0
      v2 = 0.0d0
      
      yint(1) = yint_0
 
      do j=2,npts
      
        y1 = dble(y(j-1))
        y2 = dble(y(j))
        
        cumv = cumv + (y1 + y2)*ddt_2
        v1 = v2
        v2 = cumv + dble(yint_0)  ! the yint_0 term is the constant of integration
        yint(j) = sngl(v2)
        
      end do

      return
      end
!----------------- END Trapezoidal_Integration -----------------------------


!----------------- BEGIN Acc2VD -----------------------------
      subroutine acc2vd(acc, npts, dt, rmv_trnd, vel0, dis0, vel, dis)


* Compute velocity and displacement time series from acceleration,
* assuming that the acceleration
* is represented by straight lines connecting the digitized values.

* Dates: 02/09/99 - Written by D.M. Boore
*        01/07/00 - Added initial velocity and displacements (v0, d0).
*                   This also requires the addition of a linear trend in
*                   displacement.
*                   Also, Bill Joyner, Chris Stephens, and I considered how to
*                   handle the first point.  Assuming v0 = 0, BAP uses a 
*                   trapezoidal rule such that the first vel(1) = 0.5*dt*a(1),
*                   but in acc2vd, vel(1) = 0.  In effect, BAP starts the
*                   integration at -dt rather than 0.0, with the result that
*                   because vel(1) depends on dt, vel(1) will change if the
*                   digitization interval is changed.  We agreed that this is 
*                   not correct.
*        12/18/02 - Some minor changes, such as specifying 0.0d0 rather 
*                   than 0.0 and using the function "dble" in various places
*        10/15/07 - Rename "d0" to "dis0" so as not to confuse with "d0"
*                   in "0.0d0".  For consistency, change "v0" to "vel0".
*                   More importantly, restructure the calculations so as not to
*                   lose the double-precision estimates of velocity when
*                   accumulating the displacement, and abandon the calculation 
*                   of disp based on integrating straightlines in acceleration, 
*                   using a simple trapezoidal rule instead.  These changes were
*                   made in an attempt to track down a drift in displacement
*                   for the HHE component of the KYTH recording of the
*                   Kythera earthquake recorded on a station of the EGELADOS
*                   velocity-sensor network.   The drift did not appear when I
*                   integrated the acceleration in Excel, and the SD computed 
*                   using SMC2RS also showed no indication of the drift (the
*                   long-period SD equals the peak displacement during the
*                   strong part of the shaking).   Unfortunately, the changes
*                   made today did not eliminate the drift.   My next check is
*                   to see if the 5 digit resolution in the smc files might
*                   be responsible (the data given to me by Andreas Skarlatoudis
*                   were in column files with 7 digits of resolution, with a maximum
*                   exponent of 9).  When I 
*                   opened the asc file made by smc2asc in Excel and computed dis, 
*                   I found the same trend as in the smc2vd file. This tells me
*                   that the problem is in the asc2smc conversion (but why the
*                   SD computed using the smc files shows no sign of the drift
*                   is a mystery to me).  I eventually confirmed that the problem
*                   is the limited resolution of the standard format; I have added
*                   an option to the smc utility programs to use a higher-resolution
*                   format (converting all of the utility programs will take some
*                   time---I am only doing those that I currently need right now).
!        02/06/11 - Added comments to make it clear that now use trapezoidal integration.
!                 - Delete "sngl(cumv)" and "sngl(cumd)" for vel(1) and dis(1)
!        02/07/11 - Call a subroutine to do the integration
!        06/01/11 - Delete the include statement for the integration routine; it should be placed in the programs that
!                   call acc2vd instead.  The reason for this is because
!                   the subroutine acc2vd is also included in \smsim\td_subs, and a compile
!                   error can occur if the \forprogs folder with the integration routine
!                   is not available.  This means that the trapezoidal_integration.for must be available in smsim.
!                   I guarantee this by including it in make_td_subs_file.bat, which should be called
!                   in time domain cl*.bat files.

      real acc(*), vel(*), dis(*) 
      logical rmv_trnd
      double precision cumv, cumd, a1, a2, v1, v2,
     : ddt, ddt_2, ddtdt_6

      if (rmv_trnd) then      
* remove trend first (straight line between first and last points)
* Note: acc is replaced with detrended time series
*        call dcdt(acc, dt, npts, 1, npts, .false., .true.)  ! old routine,
*                                                         ! gives steps at ends

         call rmvtrend(acc, npts)
      end if

* compute velocity and displacement

      call Trapezoidal_Integration(acc, npts, dt, vel0, vel)
      call Trapezoidal_Integration(vel, npts, dt, dis0, dis)
      
!      ddt     = dble(dt)
!      ddt_2   = 0.5d0 * dble(dt)
!      
!!      ddtdt_6 = dble(dt)*dble(dt)/6.0d0
!
!      cumv = 0.0d0
!      cumd = 0.0d0
!
!      v1 = 0.0d0
!      v2 = 0.0d0
!      
!!      vel(1) = sngl(cumv) + vel0
!      dis(1) = sngl(cumd) + dis0
!
!      vel(1) = vel0
!      dis(1) = dis0
!
!      do j=2,npts
!      
!        a1 = dble(acc(j-1))
!        a2 = dble(acc(j))
!        
!        cumv = cumv + (a1 + a2)*ddt_2
!        v1 = v2
!        v2 = cumv + dble(vel0)  ! the vel0 term is the constant of integration
!        vel(j) = sngl(v2)
!        
!        cumd = cumd + (v1 + v2)*ddt_2
!        dis(j) = sngl(cumd) + dis0  ! the dis0 term is the constant of integration
!        
!! Legacy code, when I used analytical formulas based
!! on representing the acceleration as a series of straightline segments.
!!        cumd = cumd + v1*ddt + (2.0*a1 + a2)*ddtdt_6
!        dis(j) = sngl(cumd) + dis0  ! linear term accounted for in v1*ddt
!        dis(j) = sngl(cumd) + v0*float(j-1)*dt + dis0
!
!      end do

      return
      end
!----------------- END Acc2VD -----------------------------


*----------------- BEGIN dis2va-----------------------------
       subroutine dis2va(dis, npts, dt, vel, acc) 
* Compute Acc and Vel from Dis	
* Dates: 10/28/2005 - Written by Guoquan Wang, NCAT
*        04/30/09 - modified by D. Boore, adding double precision
*                   and assuming displacement at i=0, -1 equals
*                   displacement at i=1 and at i=npts+1, npts+2 equals 
*                   displacement at i=npts (to deal with the case of a
*                   residual displacement).
*                   
	
      double precision  
     :   dm2, dm1, d0, dp1, dp2, 
     :   ddt, ddt2, ddt_2,
     :   f11, f12, f21, f22, f23
      real vel(*)
      real acc(*)
      real dis(*)
      
      ddt = dble(dt)
      ddt2 = ddt*ddt



      do i=1,npts
        vel(i)= 0.0
        acc(i)= 0.0      
      end do
        
! cal vel, centered scheme,derivation
!! Three points  First Derivative
!      do i=2,npts-1	  
!	  vel(i) = (dis(i+1)-dis(i-1))/(2.0d0*dt)
!      end do

!! Five points First Derivative

!! Five Points Second Derivative
 ! a(i)=-1/12*d(i-2)+4/3*d(i-1)-5/2*d(i)+4/3*f(i+1)-1/12*d(i+2)
 
      f11=1.0d0/12.0d0
      f12=2.0d0/3.0d0
        
      f21=-1.0d0/12.0d0
      f22=4.0d0/3.0d0
      f23=-5.0d0/2.0d0
       
      do i=3,npts-2
        dm2 = dble(dis(i-2))
        dm1 = dble(dis(i-1))
        d0  = dble(dis(i))
        dp1 = dble(dis(i+1))
        dp2 = dble(dis(i+2))
        vel(i)=sngl((f11*dm2-f12*dm1+f12*dp1-f11*dp2)/ddt)
        acc(i)=sngl((f21*dm2+f22*dm1+f23*d0+f22*dp1+f21*dp2)/ddt2)
      end do

      i = 1
        dm2 = dble(dis(1))
        dm1 = dble(dis(1))
        d0  = dble(dis(i))
        dp1 = dble(dis(i+1))
        dp2 = dble(dis(i+2))
        vel(i)=sngl((f11*dm2-f12*dm1+f12*dp1-f11*dp2)/ddt)
        acc(i)=sngl((f21*dm2+f22*dm1+f23*d0+f22*dp1+f21*dp2)/ddt2)
      i = 2
        dm2 = dble(dis(1))
        dm1 = dble(dis(i-1))
        d0  = dble(dis(i))
        dp1 = dble(dis(i+1))
        dp2 = dble(dis(i+2))
        vel(i)=sngl((f11*dm2-f12*dm1+f12*dp1-f11*dp2)/ddt)
        acc(i)=sngl((f21*dm2+f22*dm1+f23*d0+f22*dp1+f21*dp2)/ddt2)
      i = npts-1
        dm2 = dble(dis(i-2))
        dm1 = dble(dis(i-1))
        d0  = dble(dis(i))
        dp1 = dble(dis(i+1))
        dp2 = dble(dis(npts))
        vel(i)=sngl((f11*dm2-f12*dm1+f12*dp1-f11*dp2)/ddt)
        acc(i)=sngl((f21*dm2+f22*dm1+f23*d0+f22*dp1+f21*dp2)/ddt2)
      i = npts
        dm2 = dble(dis(i-2))
        dm1 = dble(dis(i-1))
        d0  = dble(dis(i))
        dp1 = dble(dis(npts))
        dp2 = dble(dis(npts))
        vel(i)=sngl((f11*dm2-f12*dm1+f12*dp1-f11*dp2)/ddt)
        acc(i)=sngl((f21*dm2+f22*dm1+f23*d0+f22*dp1+f21*dp2)/ddt2)

 
      return
      end
*----------------- END dis2va -----------------------------
* ----------------------------- BEGIN DETREND ----------------
      subroutine detrend(y, n, itype)

* itype = 1: Removes a straightline fit to first and last points, replacing
* the input array with the detrended array
* itype = 2: Remove a least squares line fit to the whole record
* itype = 3: Remove a least squares quadratic fit to the whole record
* itype = 4: Remove a least squares cubic fit to the whole record

* Dates: 10/20/09 - written by D. Boore, patterned after rvmtrend.for

      external fpoly
!      external dfpoly

      real y(*), x(n), sig(n), c(11), covar(11,11), pd(2)
      integer ic(11)
      
      npc = 11

      if (itype .eq. 1) then
        y1 = y(1)
        y2 = y(n)
        slope = (y2 - y1)/float(n-1)

        do i = 1, n
          y(i) = y(i) - (y1 + slope*float(i-1))
        end do
        return
      end if

      do i = 1, n
        x(i) = float(i-1)
        sig(i) = 1.0
      end do
      
*   Now fit the curve:
     
      nc = iabs(itype-1) + 1
      
      c = 0.0

      do i = 1, nc
        ic(i)     = 1
      end do                

      call lfit(x, y, sig, n, c, ic, nc,
     :     covar, npc, chisq, fpoly)
!      call dlfit(x, y, sig, n, c, ic, nc,
!     :     covar, npc, chisq, dfpoly)
     
      nd = 2
     
      do i = 1, n
        xeval = float(i-1)
        call ddpoly(c, nc, xeval, pd, nd)
!        call dddpoly(c, nc, xeval, pd, nd)
        y(i) = y(i) - pd(1)
      end do

      
      return
      end
* ----------------------------- END DETREND ----------------

!      include '\forprogs\lfit.for'
!      include '\forprogs\fpoly.for'
!      include '\forprogs\ddpoly.for'
!      include '\forprogs\covsrt.for'
!      include '\forprogs\gaussj.for'
      

! ----------------------------- BEGIN RMVTREND ----------------
      subroutine rmvtrend(y, n)

! Removes a straightline fit to first and last points, replacing
! the input array with the detrended array

! Dates: 02/09/99 - written by D. Boore
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)


      real y(*)

      y1 = y(1)
      y2 = y(n)
      slope = (y2 - y1)/float(n-1)

      do i = 1, n
        y(i) = y(i) - (y1 + slope*float(i-1))
      end do

      return
      end
! ----------------------------- END RMVTREND ----------------


* ------------------------------------------------------------------- lfit
      SUBROUTINE lfit(x,y,sig,ndat,a,ia,ma,covar,npc,chisq,funcs)
      INTEGER ma,ia(ma),npc,ndat,MMAX
      REAL chisq,a(ma),covar(npc,npc),sig(ndat),x(ndat),y(ndat)
      EXTERNAL funcs
      PARAMETER (MMAX=50)
CU    USES covsrt,gaussj
      INTEGER i,j,k,l,m,mfit
      REAL sig2i,sum,wt,ym,afunc(MMAX),beta(MMAX)
      mfit=0
      do 11 j=1,ma
        if(ia(j).ne.0) mfit=mfit+1
11    continue
      if(mfit.eq.0) pause 'lfit: no parameters to be fitted'
      do 13 j=1,mfit
        do 12 k=1,mfit
          covar(j,k)=0.
12      continue
        beta(j)=0.
13    continue
      do 17 i=1,ndat
        call funcs(x(i),afunc,ma)
        ym=y(i)
        if(mfit.lt.ma) then
          do 14 j=1,ma
            if(ia(j).eq.0) ym=ym-a(j)*afunc(j)
14        continue
        endif
        sig2i=1./sig(i)**2
        j=0
        do 16 l=1,ma
          if (ia(l).ne.0) then
            j=j+1
            wt=afunc(l)*sig2i
            k=0
            do 15 m=1,l
              if (ia(m).ne.0) then
                k=k+1
                covar(j,k)=covar(j,k)+wt*afunc(m)
              endif
15          continue
            beta(j)=beta(j)+ym*wt
          endif
16      continue
17    continue
      do 19 j=2,mfit
        do 18 k=1,j-1
          covar(k,j)=covar(j,k)
18      continue
19    continue
      call gaussj(covar,mfit,npc,beta,1,1)
      j=0
      do 21 l=1,ma
        if(ia(l).ne.0) then
          j=j+1
          a(l)=beta(j)
        endif
21    continue
      chisq=0.
      do 23 i=1,ndat
        call funcs(x(i),afunc,ma)
        sum=0.
        do 22 j=1,ma
          sum=sum+a(j)*afunc(j)
22      continue
        chisq=chisq+((y(i)-sum)/sig(i))**2
23    continue
      call covsrt(covar,npc,ma,ia,mfit)
      return
      END
* ------------------------------------------------------------------- lfit
* -------------------------------------------------------------------- dlfit
      SUBROUTINE dlfit(x,y,sig,ndat,a,ia,ma,covar,npc,chisq,funcs)
      INTEGER ma,ia(ma),npc,ndat,MMAX
      DOUBLE PRECISION chisq,a(ma),covar(npc,npc),sig(ndat),
     :  x(ndat),y(ndat)
      EXTERNAL funcs
      PARAMETER (MMAX=50)
CU    USES dcovsrt,dgaussj
      INTEGER i,j,k,l,m,mfit
      DOUBLE PRECISION sig2i,sum,wt,ym,afunc(MMAX),beta(MMAX)
      mfit=0
      do 11 j=1,ma
        if(ia(j).ne.0) mfit=mfit+1
11    continue
      if(mfit.eq.0) pause 'lfit: no parameters to be fitted'
      do 13 j=1,mfit
        do 12 k=1,mfit
          covar(j,k)=0.d0
12      continue
        beta(j)=0.d0
13    continue
      do 17 i=1,ndat
        call funcs(x(i),afunc,ma)
        ym=y(i)
        if(mfit.lt.ma) then
          do 14 j=1,ma
            if(ia(j).eq.0) ym=ym-a(j)*afunc(j)
14        continue
        endif
        sig2i=1.d0/sig(i)**2
        j=0
        do 16 l=1,ma
          if (ia(l).ne.0) then
            j=j+1
            wt=afunc(l)*sig2i
            k=0
            do 15 m=1,l
              if (ia(m).ne.0) then
                k=k+1
                covar(j,k)=covar(j,k)+wt*afunc(m)
              endif
15          continue
            beta(j)=beta(j)+ym*wt
          endif
16      continue
17    continue
      do 19 j=2,mfit
        do 18 k=1,j-1
          covar(k,j)=covar(j,k)
18      continue
19    continue
      call dgaussj(covar,mfit,npc,beta,1,1)
      j=0
      do 21 l=1,ma
        if(ia(l).ne.0) then
          j=j+1
          a(l)=beta(j)
        endif
21    continue
      chisq=0.d0
      do 23 i=1,ndat
        call funcs(x(i),afunc,ma)
        sum=0.d0
        do 22 j=1,ma
          sum=sum+a(j)*afunc(j)
22      continue
        chisq=chisq+((y(i)-sum)/sig(i))**2
23    continue
      call dcovsrt(covar,npc,ma,ia,mfit)
      return
      END
* -------------------------------------------------------------------- dlfit

* ----------------------------------------------------------- dgaussj 
      SUBROUTINE dgaussj(a,n,np,b,m,mp)
      INTEGER m,mp,n,np,NMAX
      DOUBLE PRECISION a(np,np),b(np,mp)
      PARAMETER (NMAX=50)
      INTEGER i,icol,irow,j,k,l,ll,indxc(NMAX),indxr(NMAX),ipiv(NMAX)
      DOUBLE PRECISION big,dum,pivinv
      do 11 j=1,n
        ipiv(j)=0
11    continue
      do 22 i=1,n
        big=0.d0
        do 13 j=1,n
          if(ipiv(j).ne.1)then
            do 12 k=1,n
              if (ipiv(k).eq.0) then
                if (dabs(a(j,k)).ge.big)then
                  big=dabs(a(j,k))
                  irow=j
                  icol=k
                endif
              else if (ipiv(k).gt.1) then
                pause 'singular matrix in gaussj'
              endif
12          continue
          endif
13      continue
        ipiv(icol)=ipiv(icol)+1
        if (irow.ne.icol) then
          do 14 l=1,n
            dum=a(irow,l)
            a(irow,l)=a(icol,l)
            a(icol,l)=dum
14        continue
          do 15 l=1,m
            dum=b(irow,l)
            b(irow,l)=b(icol,l)
            b(icol,l)=dum
15        continue
        endif
        indxr(i)=irow
        indxc(i)=icol
        if (a(icol,icol).eq.0.d0) pause 'singular matrix in gaussj'
        pivinv=1.d0/a(icol,icol)
        a(icol,icol)=1.d0
        do 16 l=1,n
          a(icol,l)=a(icol,l)*pivinv
16      continue
        do 17 l=1,m
          b(icol,l)=b(icol,l)*pivinv
17      continue
        do 21 ll=1,n
          if(ll.ne.icol)then
            dum=a(ll,icol)
            a(ll,icol)=0.d0
            do 18 l=1,n
              a(ll,l)=a(ll,l)-a(icol,l)*dum
18          continue
            do 19 l=1,m
              b(ll,l)=b(ll,l)-b(icol,l)*dum
19          continue
          endif
21      continue
22    continue
      do 24 l=n,1,-1
        if(indxr(l).ne.indxc(l))then
          do 23 k=1,n
            dum=a(k,indxr(l))
            a(k,indxr(l))=a(k,indxc(l))
            a(k,indxc(l))=dum
23        continue
        endif
24    continue
      return
      END
* ----------------------------------------------------------- dgaussj 
      SUBROUTINE gaussj(a,n,np,b,m,mp)
      INTEGER m,mp,n,np,NMAX
      REAL a(np,np),b(np,mp)
      PARAMETER (NMAX=50)
      INTEGER i,icol,irow,j,k,l,ll,indxc(NMAX),indxr(NMAX),ipiv(NMAX)
      REAL big,dum,pivinv
      do 11 j=1,n
        ipiv(j)=0
11    continue
      do 22 i=1,n
        big=0.
        do 13 j=1,n
          if(ipiv(j).ne.1)then
            do 12 k=1,n
              if (ipiv(k).eq.0) then
                if (abs(a(j,k)).ge.big)then
                  big=abs(a(j,k))
                  irow=j
                  icol=k
                endif
              endif
12          continue
          endif
13      continue
        ipiv(icol)=ipiv(icol)+1
        if (irow.ne.icol) then
          do 14 l=1,n
            dum=a(irow,l)
            a(irow,l)=a(icol,l)
            a(icol,l)=dum
14        continue
          do 15 l=1,m
            dum=b(irow,l)
            b(irow,l)=b(icol,l)
            b(icol,l)=dum
15        continue
        endif
        indxr(i)=irow
        indxc(i)=icol
        if (a(icol,icol).eq.0.) pause 'singular matrix in gaussj'
        pivinv=1./a(icol,icol)
        a(icol,icol)=1.
        do 16 l=1,n
          a(icol,l)=a(icol,l)*pivinv
16      continue
        do 17 l=1,m
          b(icol,l)=b(icol,l)*pivinv
17      continue
        do 21 ll=1,n
          if(ll.ne.icol)then
            dum=a(ll,icol)
            a(ll,icol)=0.
            do 18 l=1,n
              a(ll,l)=a(ll,l)-a(icol,l)*dum
18          continue
            do 19 l=1,m
              b(ll,l)=b(ll,l)-b(icol,l)*dum
19          continue
          endif
21      continue
22    continue
      do 24 l=n,1,-1
        if(indxr(l).ne.indxc(l))then
          do 23 k=1,n
            dum=a(k,indxr(l))
            a(k,indxr(l))=a(k,indxc(l))
            a(k,indxc(l))=dum
23        continue
        endif
24    continue
      return
      END

* ------------------------------------------------------------------ covsrt
      SUBROUTINE covsrt(covar,npc,ma,ia,mfit)
      INTEGER ma,mfit,npc,ia(ma)
      REAL covar(npc,npc)
      INTEGER i,j,k
      REAL swap
      do 12 i=mfit+1,ma
        do 11 j=1,i
          covar(i,j)=0.
          covar(j,i)=0.
11      continue
12    continue
      k=mfit
      do 15 j=ma,1,-1
        if(ia(j).ne.0)then
          do 13 i=1,ma
            swap=covar(i,k)
            covar(i,k)=covar(i,j)
            covar(i,j)=swap
13        continue
          do 14 i=1,ma
            swap=covar(k,i)
            covar(k,i)=covar(j,i)
            covar(j,i)=swap
14        continue
          k=k-1
        endif
15    continue
      return
      END
* ------------------------------------------------------------------ covsrt

* ------------------------------------------------------------------- ddpoly
      SUBROUTINE ddpoly(c,nc,x,pd,nd)
      INTEGER nc,nd
      REAL x,c(nc),pd(nd)
      INTEGER i,j,nnd
      REAL const
      pd(1)=c(nc)
      do 11 j=2,nd
        pd(j)=0.
11    continue
      do 13 i=nc-1,1,-1
        nnd=min(nd,nc+1-i)
        do 12 j=nnd,2,-1
          pd(j)=pd(j)*x+pd(j-1)
12      continue
        pd(1)=pd(1)*x+c(i)
13    continue
      const=2.
      do 14 i=3,nd
        pd(i)=const*pd(i)
        const=const*i
14    continue
      return
      END
* ------------------------------------------------------------------- ddpoly

! -----------------------------------------------------------------  fpoly
      SUBROUTINE fpoly(x,p,np)
      INTEGER np
      REAL x,p(np)
      INTEGER j
      p(1)=1.
      do 11 j=2,np
        p(j)=p(j-1)*x
11    continue
      return
      END
! -----------------------------------------------------------------  fpoly

* ----------------------------------------------------------------- dcovsrt
      SUBROUTINE dcovsrt(covar,npc,ma,ia,mfit)
      INTEGER ma,mfit,npc,ia(ma)
      DOUBLE PRECISION covar(npc,npc)
      INTEGER i,j,k
      DOUBLE PRECISION swap
      do 12 i=mfit+1,ma
        do 11 j=1,i
          covar(i,j)=0.d0
          covar(j,i)=0.d0
11      continue
12    continue
      k=mfit
      do 15 j=ma,1,-1
        if(ia(j).ne.0)then
          do 13 i=1,ma
            swap=covar(i,k)
            covar(i,k)=covar(i,j)
            covar(i,j)=swap
13        continue
          do 14 i=1,ma
            swap=covar(k,i)
            covar(k,i)=covar(j,i)
            covar(j,i)=swap
14        continue
          k=k-1
        endif
15    continue
      return
      END
* ----------------------------------------------------------------- dcovsrt

* ------------------------------------------------------------------ dddpoly
      SUBROUTINE dddpoly(c,nc,x,pd,nd)
      INTEGER nc,nd
      DOUBLE PRECISION x,c(nc),pd(nd)
      INTEGER i,j,nnd
      DOUBLE PRECISION const
      pd(1)=c(nc)
      do 11 j=2,nd
        pd(j)=0.d0
11    continue
      do 13 i=nc-1,1,-1
        nnd=min(nd,nc+1-i)
        do 12 j=nnd,2,-1
          pd(j)=pd(j)*x+pd(j-1)
12      continue
        pd(1)=pd(1)*x+c(i)
13    continue
      const=2.d0
      do 14 i=3,nd
        pd(i)=const*pd(i)
        const=const*i
14    continue
      return
      END
* ------------------------------------------------------------------ dddpoly

* -------------------------------------------------------------- dfpoly
      SUBROUTINE dfpoly(x,p,np)
      INTEGER np
      DOUBLE PRECISION x,p(np)
      INTEGER j
      p(1)=1.d0
      do 11 j=2,np
        p(j)=p(j-1)*x
11    continue
      return
      END
* -------------------------------------------------------------- dfpoly
* ----------------------------------------------------------- Begin GetF_Out
      subroutine GetF_Out(f_in, ext_nu, string, insert, 
     :                    ipos, f_out)

* Create the output file name.

* - If ext_nu .ne. ' ' .and. ext_nu .ne. 'XXX', replaces input extension 
*   with ext_nu
* - Either inserts or overwrites string_c at position ipos
* - Note that if inserted, it is inserted to the left of the indicated 
* position.  Here are some examples:
*      ipos     inserted where?
*       0          to left of period
*      -1          to left of stem character next to period
*      +1          to left of first character of extension (note
*                  that the extension will always be <= 3 characters, so
*                  inserting in the extension will push characters off the end.
*                  Likewise, the stem is always <= 8 characters, so inserting
*                  can push characters off the end of the stem.


* Dates: 11/07/01 - Written by D. Boore

      character f_in*(*), ext_nu*(*), string*(*), f_out*(*) 
      character ext_in*3, ext_nu_upper*3
      character stem*20, stem_in*8, ext*10

      logical insert

      call trim_c(f_in,nc_f_in)
      call trim_c(string, nc_string)
      call trim_c(ext_nu,nc_ext_nu)
      ext_nu_upper = ' '
      if (nc_ext_nu .ne. 0) then
        ext_nu_upper = ext_nu(1:nc_ext_nu)
        call trim_c(ext_nu_upper,nc_ext_nu_upper)
        call upstr(ext_nu_upper)
      end if

      indp = index(f_in,'.')
      stem_in = ' '
      stem_in = f_in(1:indp-1)
      call trim_c(stem_in,nc_stem_in)
      ext_in = ' '
      ext_in = f_in(indp+1:nc_f_in)
      call trim_c(ext_in, nc_ext_in)

      stem = ' '
      stem = stem_in
      nc_stem = nc_stem_in

      ext = ' '
      ext = ext_in
      nc_ext = nc_ext_in

* Check if new extension is to be used:      
      if(ext_nu_upper .ne. ' ' .and. ext_nu_upper .ne. 'XXX') then
        ext = ext_nu(1:nc_ext_nu)
        call trim_c(ext,nc_ext)
* Have to reset ext_in; it could have changed length
        ext_in = ' '
        ext_in = ext        
        call trim_c(ext_in,nc_ext_in)
      end if

* Insert or overwrite string if nonzero length
      if (nc_string .ne. 0) then  ! Not finished
        if (nc_stem_in + ipos .lt. 0) ipos = -nc_stem_in !reset to 1rst ch
        if (.not. insert) then  ! overwrite
          if (ipos .le. 0) then  ! work on stem
            stem(nc_stem_in+ipos+1:nc_stem_in+ipos+nc_string) = 
     :        string(1:nc_string)
          else ! work on extension
            ext(ipos:ipos+nc_string-1) = string(1:nc_string)
          end if
        else  ! insert
          if (ipos .le. 0) then  ! work on stem
            if (nc_stem_in+ipos .gt. 0)  ! some original to left of insert
     :      stem(1:nc_stem_in+ipos) = stem_in(1:nc_stem_in+ipos)
            stem(nc_stem_in+ipos+1:nc_stem_in+ipos+nc_string) = 
     :        string(1:nc_string)
            if (ipos .lt. 0)             ! some original to right of insert
     :      stem(nc_stem_in+ipos+nc_string+1:nc_stem_in+nc_string) =
     :        stem_in(nc_stem_in+ipos+1:nc_stem_in)
          else ! work on extension
            if (ipos-1 .gt. 0)           ! some original to left of insert
     :      ext(1:ipos-1) = ext_in(1:ipos-1)
            ext(ipos:ipos+nc_string-1)= string(1:nc_string)
            if (ipos .lt. nc_ext_in)
     :      ext(ipos+nc_string:nc_string+nc_ext_in) = 
     :        ext_in(ipos:nc_ext_in)     ! some original to right of insert
          end if
        end if
      end if

      call trim_c(stem,nc_stem)
      call trim_c(ext,nc_ext)
      indstem = min0(nc_stem,8)
      indext  = min0(nc_ext,3)
      f_out = stem(1:indstem)//'.'//ext(1:indext)
      call trim_c(f_out,nc_f_out)
      if (f_out(1:nc_f_out) .eq. f_in(1:nc_f_in)) then
        write(*,'(a)') '  Warning, no change in filename, so '//
     :                   'f_out = xxxxxxxx.xxx'
         f_out = 'xxxxxxxx.xxx'
      end if

      return
      end

* ----------------------------------------------------------- End GetF_Out
!--------------------- BEGIN GET_DATE  ---------------------
      subroutine get_date(date_c)

! Dates: 04/29/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)


      implicit none


      character date_c*(*)
!      integer*2 iyr, imon, iday
      character datx*8, timx*10


! Microsoft compiler version:
!      call GETDAT (iyr, imon, iday)


!      date_c = ' '         ! character date_c*10


!      date_c(3:3) = '/'
!      date_c(6:6) = '/'


!      write(date_c(1:2),'(i2.2)') imon      
!      write(date_c(4:5),'(i2.2)') iday      
!      write(date_c(7:10),'(i4.4)') iyr      


! Lahey compiler version:
!      call DATE(date_c)   ! character date_c*8, but *10 is OK


! Standard Fortran 90 intrinsic Subroutine DATE_AND_TIME
      call DATE_AND_TIME( datx, timx )
! Date is returned as 'CCYYMMDD'
! Time is returned as 'hhmmss.sss'
! date_c should be declared as
! character date_c*10
      date_c = ' '
      date_c = datx(5:6) // '/' // datx(7:8) // '/' // datx(1:4)


      return
      end
!--------------------- END GET_DATE  ---------------------!--------------------- BEGIN GET_TIME  ---------------------
      subroutine get_time(time_c)

! Dates: 04/29/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      implicit none


      character time_c*(*)
!      integer*2 ihr, imin, isec, i100th
      character datx*8, timx*10


! Microsoft compiler version:
!      call GETTIM (ihr, imin, isec, i100th)


!      time_c = ' '             ! character time_c*11 for both compilers


!      time_c(3:3) = ':'
!      time_c(6:6) = ':'
!      time_c(9:9) = '.'


!      write(time_c(1:2),'(i2.2)') ihr      
!      write(time_c(4:5),'(i2.2)') imin      
!      write(time_c(7:8),'(i2.2)') isec      
!      write(time_c(10:11),'(i2.2)') i100th


! Lahey compiler version:
!      call TIME(time_c)


! Standard Fortran 90 intrinsic Subroutine DATE_AND_TIME
      call DATE_AND_TIME( datx, timx )
! Date is returned as 'CCYYMMDD'
! Time is returned as 'hhmmss.sss'
! time_c should be declared as
! character time_c*11
      time_c = ' '
      time_c = timx(1:2)//':'//timx(3:4) // ':' // timx(5:9)


      return
      end
!--------------------- END GET_TIME  --------------------- ! --------------------------------------------------- Get_Path_From_File_Name
      subroutine get_path_from_file_name(f_in,nc_f_in,path,nc_path)

! Extract path from a file name (nc_path = 0 if no path)

! Dates: 06/08/01 - Written by D. Boore
!        07/16/07 - Renamed from "get_path.for", because
!                   I want to use that name for a new routine
!                   that obtains the path name from a system call
!        11/07/16 - Some small changes, such as including length of f_in in 
!                   nc_path = index(f_in(1:nc_f_in),'\') statement

      character f_in*(*), path*(*)

      path = ' '
      nc_path = index(f_in(1:nc_f_in),'\') 
      if (nc_path == 0) then  ! no path
        return
      end if

! Find last occurrence of '\':

      do i = nc_f_in, 1, -1
        if (f_in(i:i) == '\') then
          nc_path = i
          path = f_in(1:nc_path)
          return
        end if
      end do
                
      write(*,'(a)') '  Oops!  Expected path in '//f_in(1:nc_f_in)//
     : ' but none found'

      return
      end
! --------------------------------------------------- Get_Path_From_File_Name
! ------------------------------------------------------------------ Get_Avg
      subroutine Get_Avg(a, 
     :  n_start, n_stop, npts, sps, icorr_cumavg, avg)

! Find mean of a between the two indices

! If itype = 1, then half weight is given to the first and last points
! (as it would be if computing velocity when the data before and after the
! indices are 0.0).

! Also consider (no confidence limits, however):

!      SUBROUTINE avg_std(data,n,avg,std,itype)
      
! Computes mean and standard deviation for an array of data.
! This is based on the Numerical Recipes program avevar.
! Itype specifies the type of mean:
! itype = 1: arithmetic mean, std = standard deviation
! itype = 2: geometric mean, std = factor by which mean is divided
!                                  and multiplied to yield the
!                                  equivalent of minus and plus
!                                  one standard deviation

! Or

!      subroutine mean_std_cl(a,npts,mean,std,seom,cl70,cl90,cl95,cl99)
! Computes mean, standard deviation, and confidence limits of the array entries





! Dates: 12/17/02 - Written by D. Boore
!        12/18/02 - Add icorr_cumavg to try to get program smc_pad to give 0.0
!                   velocity at the end of the data segment
!        12/18/02 - Rethinking computation of average when want to
!                   apply it to velocity case (looking at acc2vd.for algorithm)
!        05/01/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      real a(*)
      real*8 cum_avg

      avg = 0.0
!      if (t4mean_start .lt. 0.0) t4mean_start = 0.0
!      if (t4mean_stop .gt. tend) t4mean_stop = tend
!      n_start = nint(sps * t4mean_start + 1.1)
!      n_stop = nint(sps * t4mean_stop + 1.1)
      if (n_start .lt. 0)    n_start = 1
      if (n_stop .gt. npts) n_stop = npts
      n_4mean = n_stop - n_start + 1
!      t4mean = float(n_4mean -1)/sps
      cum_avg = 0.0
      do i = n_start, n_stop
        cum_avg = cum_avg + dble(a(i))
      end do

      if (icorr_cumavg .eq. 1) then  ! Effectively ignore icorr_cumavg now
!        cum_avg = cum_avg - 0.5d00*(dble(a(n_start))+dble(a(n_stop)))         
!        avg = sngl(cum_avg/dble(float(n_4mean-1)))
        avg = sngl(cum_avg/dble(float(n_4mean)))
      else
        avg = sngl(cum_avg/dble(float(n_4mean)))
      end if

      return
      end
! ------------------------------------------------------------------ Get_Avg
! ------------------------------------------------------  begin construct_filename_extension
      subroutine construct_filename_extension
     :      (tskip, tlength, option, 
     :       flc, nslope_lc, icaus_lc,
     :       fhc, nslope_hc, icaus_hc, icaus_bn,
     :       filter_type, nc_filter_type,
     :       taper_beginning, taper_end, 
     :       fn_ext_c, nc_fn_ext_c)
     
! Construct a filename extension based on the blpadflt parameter
! "option" (10=no baseline) and the blpadflt filtering  
! parameters.  At this time I do not include baseline correction or
! tapering details, as these are less commonly used than the filtering

! Dates: 12/08/09 - Add taper info to filename
!        01/25/10 - Correct error in construct_filename_extension.for when auto naming the 
!                   output file if only remove mean (no filtering).
!        03/11/10 - Correct error of not calling trim_c for fn_ext_c
!        03/20/10 - Allow for a notch filter.
!        12/04/10 - Add tskip, tlength to name
!        05/16/14 - Modernize boolean statements
!                 - Introduce option==11 to mean no baseline, and no mean removed.
!                   I needed this in order to make the filename of the unfiltered velocity
!                   time series in the filt_plot_lc output indicate correctly whether
!                   or not a mean had been removed.
!        07/11/17 - Add more filter possibilities
!        07/13/17 - Determine filter type in calling program
!        01/09/20 - Change portion of automatic filename for baseline correction from 
!                   "blc" to "v0_blc" for option 3.
     
      character fn_ext_c*(*), d_c*6, freql_c*6, freqh_c*6, ns_c*2
      integer option
      character filter_type*(*)
      
      fn_ext_c = ' '
      fn_ext_c = '.tskp'

      call trim_c(fn_ext_c, nc_fn_ext_c)
      if (tskip < 10.0) then
        write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+3),
     :             '(f3.1)') tskip
      else if (tskip < 100.0) then
        write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+4),
     :             '(f4.1)') tskip
      else if (tskip < 1000.0) then
        write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+5),
     :             '(f5.1)') tskip
      else if (tskip < 10000.0) then
        write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+6),
     :             '(f6.1)') tskip
      else 
        write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+7),
     :             '(f7.1)') tskip
      end if
      call trim_c(fn_ext_c, nc_fn_ext_c)
      
      fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//'.tread'
      call trim_c(fn_ext_c, nc_fn_ext_c)
      if (tlength <  0.0) then
        write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+4),
     :             '(f4.1)') tlength
      else if (tlength < 10.0) then
        write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+3),
     :             '(f3.1)') tlength
      else if (tlength < 100.0) then
        write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+4),
     :             '(f4.1)') tlength
      else if (tlength < 1000.0) then
        write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+5),
     :             '(f5.1)') tlength
      else if (tlength < 10000.0) then
        write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+6),
     :             '(f6.1)') tlength
      else 
        write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+7),
     :             '(f7.1)') tlength
      end if
      call trim_c(fn_ext_c, nc_fn_ext_c)
     
      if (icaus_lc == 1 .and. icaus_hc == 1) then
        icaus = 1
      else
        icaus = 0
        end if


      
      if (filter_type(1:nc_filter_type) == 'NF') then  ! no filter
        if (option == 10 .or. option == 11) then ! no baseline correction
          if (option == 11) then
            fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//'.xrm'
          else
            fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//'.rm'
          end if
        else  ! baseline correction
          if (option == 3) then          
            fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//'.v0_blc'
          else
            fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//'.blc'
          end if
        end if
        call trim_c(fn_ext_c, nc_fn_ext_c)

      else ! filtering
      
        if (filter_type(1:nc_filter_type) /= 'NF') then  ! A filter has been applied

          if (filter_type(1:nc_filter_type) == 'LC') then ! locut only
            ns_c = ' '
            write(ns_c(1:2),'(i2.2)') nslope_lc
            call freq2char(flc, freql_c)
            if (icaus_lc == 0) then
              fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :         '.alc'//freql_c(1:6)//'ns'//ns_c(1:2)
            else
              fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :         '.clc'//freql_c(1:6)//'ns'//ns_c(1:2)
            end if
          else if (filter_type(1:nc_filter_type) == 'HC') then ! hicut only
            ns_c = ' '
            write(ns_c(1:2),'(i2.2)') nslope_hc
            call freq2char(fhc, freqh_c)
            if (icaus_hc == 0) then
              fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :         '.ahc'//freqh_c(1:6)//'ns'//ns_c(1:2)
            else
              fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :         '.chc'//freqh_c(1:6)//'ns'//ns_c(1:2)
            end if
          else if (filter_type(1:nc_filter_type) == 'NOTCH') then ! notch
            d = real(nslope_lc)*10.0**(-real(nslope_hc))
            call freq2char(flc, freql_c)
            call freq2char(d, d_c)
            if (icaus_bn == 1) then  !causal
              fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :       '.cnotch'//freql_c(1:6)//'d'//d_c(1:6)
            else
              fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :       '.anotch'//freql_c(1:6)//'d'//d_c(1:6)
            end if
          else if (filter_type(1:nc_filter_type) == 'BAND') then ! bandpass          
            nslope_bp = max0(nslope_lc, nslope_hc)
            ns_c = ' '
            write(ns_c(1:2),'(i2.2)') nslope_bp
            call freq2char(flc, freql_c)
            call freq2char(fhc, freqh_c)
            if (icaus_bn == 0) then
              fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :       '.abp'//freql_c(1:6)//'-'//freqh_c(1:6)//'ns'//ns_c(1:2)
            else
                fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :       '.cbp'//freql_c(1:6)//'-'//freqh_c(1:6)//'ns'//ns_c(1:2)
            end if
          else if (filter_type(1:nc_filter_type) == 'LCHC') then ! locut followed by hicut
            ns_c = ' '
            write(ns_c(1:2),'(i2.2)') nslope_lc
            call freq2char(flc, freql_c)
            if (icaus_lc == 0) then
              fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :         '.alc'//freql_c(1:6)//'ns'//ns_c(1:2)
            else
              fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :         '.clc'//freql_c(1:6)//'ns'//ns_c(1:2)
            end if
            call trim_c(fn_ext_c, nc_fn_ext_c)
            ns_c = ' '
            write(ns_c(1:2),'(i2.2)') nslope_hc
            call freq2char(fhc, freqh_c)
            if (icaus_hc == 0) then
              fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :         '.ahc'//freqh_c(1:6)//'ns'//ns_c(1:2)
            else
              fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//
     :         '.chc'//freqh_c(1:6)//'ns'//ns_c(1:2)
            end if
          else
            write(*,'(a)') 
     :          ' In construct_filename_extension, '//
     :          'unknown filter type; quitting!!! '
            stop
          end if
        end if

        call trim_c(fn_ext_c, nc_fn_ext_c)

! Add taper info if an acausal filter:
 
        if (icaus_bn == 0 .or. icaus_lc == 0 .or. icaus_hc == 0) then 
      
          fn_ext_c = fn_ext_c(1:nc_fn_ext_c)//'.taper_'
          call trim_c(fn_ext_c, nc_fn_ext_c)
        
          if (taper_beginning >= 10.0) then
            write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+4),
     :             '(f4.1)') taper_beginning
          else
            fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+1) = '_'
            write(fn_ext_c(nc_fn_ext_c+2:nc_fn_ext_c+4),
     :             '(f3.1)') taper_beginning
          end if
        
          fn_ext_c(nc_fn_ext_c+5:nc_fn_ext_c+5) = '_'
          call trim_c(fn_ext_c, nc_fn_ext_c)
          if (taper_end >= 10.0) then
            write(fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+4),
     :             '(f4.1)') taper_end
          else
            fn_ext_c(nc_fn_ext_c+1:nc_fn_ext_c+1) = '_'
            write(fn_ext_c(nc_fn_ext_c+2:nc_fn_ext_c+4),
     :             '(f3.1)') taper_end
          end if
 
          call trim_c(fn_ext_c, nc_fn_ext_c)
        end if

      end if 
      
      call trim_c(fn_ext_c, nc_fn_ext_c)
      
      return
      end
! ------------------------------------------------------  end construct_filename_extension

      
! ------------------------------------------------------  begin freq2char
      subroutine freq2char(f, freq_c)
        character freq_c*6
        real f
        freq_c = '000000'
        if (f.lt.10.0) then
          write(freq_c(2:6),'(f5.3)') f
        else  
          write(freq_c(1:6),'(f6.3)') f
        end if
        
      return
      end
! ------------------------------------------------------  end freq2char       
